<?php

if (!function_exists("get_option")) die("Access denied.");

require_once(EZSCM_PATH . "class.ezscm_backend.php");
$ezscm = new Ezscm_backend();

parse_str($_REQUEST["data"], $data);
$action = $data["action"];
$id 	= isset($data["id"]) ? $data["id"] : null;
$s_id 	= isset($data["s_id"]) ? $data["s_id"] : null;

// verify nonce
if (!wp_verify_nonce($data["nonce"], "ezscm-nonce")) {
	ezscm_send_ajax(array("error" => "Could not verify security nonce. Please refresh this page."));
	die();
}

switch ($action) {
	// add schedule
	case "add":
		ezscm_send_ajax($ezscm->schedule_add());
	break;

	// delete entry
	case "entry_delete":
		ezscm_send_ajax($ezscm->entry_delete($id));
	break;

	// get single entry
	case "get_entry":
		ezscm_send_ajax($ezscm->get_entry($id));
	break;

	// get schedule entries
	case "get_entries":
		ezscm_send_ajax($ezscm->get_entries($s_id));
	break;

	// get schedule entries and options
	case "get_schedule":
		$ret = array(
			"entries" => $ezscm->get_entries($id, $data["week"]),
			"options" => $ezscm->get_schedule_settings($id)
		);

		ezscm_send_ajax($ret);
	break;

	// rename schedule
	case "rename":
		ezscm_send_ajax($ezscm->schedule_rename($id, $data["name"]));
	break;

	// save entry
	case "save_entry":
		$res = $ezscm->save_entry($data);

		// failed to insert entry
		if (isset($res["error"])) {
			ezscm_send_ajax($res);

			die();
		}

		// return saved entry
		ezscm_send_ajax($ezscm->get_entry($res["id"]));
	break;

	// clear schedule
	case "schedule_clear":
		ezscm_send_ajax($ezscm->schedule_clear($id));
	break;

	// delete schedule
	case "schedule_del":
		ezscm_send_ajax($ezscm->schedule_del($id));
	break;

	// update options
	case "update_options":
		$ezscm->update_settings_schedule($data["opt"], $s_id);

		$ret = array(
			"entries" => $ezscm->get_entries($s_id),
			"options" => $ezscm->get_schedule_settings($s_id)
		);

		ezscm_send_ajax($ret);
	break;
}

die();


/*function ezscm_send_ajax($msg) {
	//error_log(var_export($msg, true));
	if (is_object($msg) && property_exists($msg, "data")) $msg->data = json_decode($msg->data);
	$ret = json_encode($msg);
	//error_log(var_export($ret, true));

	echo $ret;
}*/

function ezscm_send_ajax($msg) {
	// check for errors in array
	if (is_array($msg)) {
		foreach ($msg as $m) {
			if (is_array($m) && isset($m["error"])) {
				echo json_encode($m);

				return;
			}
		}
	}

	echo json_encode($msg);
}

?>