var ezscm_backend = function($) {
	$.fx.speeds._default = 50;

	/**
		ui
	**/
	// default date format
	date_format_internal = "YYYY-MM-DD";

	datepicker_defaults = {
		changeMonth: true,
		changeYear: true,
		dateFormat: date_format_internal,
		showOtherMonths: true,
		selectOtherMonths: true,
		showWeek: true,
		firstDay: 1
	};

	// init ui
	ui_init();
	// tabs
	$("#tabs").tabs();

	// dialogs
	var dialog_default_attr = {
		autoOpen: false,
		height: Math.min(800, $(window).height() - 200),
		width: Math.min(1200, $(window).width() - 200),
		modal: true
	};

	var dialog_options_buttons = {
		buttons: {
			"Update options": function() {
				$(".ezscm-option-save").click();
			},
			"Cancel": function() {
				$(this).dialog("close");
			}
		}
	};
	$(".ezscm-options-dialog").dialog($.extend(dialog_default_attr, dialog_options_buttons));

	/**
		ajax actions
	**/
	$("body").on("click", "[data-action]", function() {
		do_action($(this));

		var selectgroup = $(this).data("selectgroup");
		if (selectgroup) {
			$(".button-primary[data-selectgroup='" + selectgroup + "']").removeClass("button-primary");
			$(this).addClass("button-primary");
		}

		return false;
	});

	// toggle single entry data
	$("body").on("click", ".ezscm-entry-header", function() {
		$(this).parents(".ezscm-entry-single").find(".ezscm-entry-data").toggle();
	});

	/**
		entry selected
	**/
	$("body").on("click", ".schedule-entry", function() {
		$(".entry-selected").removeClass("entry-selected");
		$(this).addClass("entry-selected");

		if ($(this).hasClass("entry-free")) {
			$(".entry-details").removeClass("hidden");
			var date_internal = $(this).data("date");
			var time_internal = $(this).data("time");

			var entry = {
				date_internal: date_internal,
				date:          moment(date_internal).format("MM-DD-YYYY"),
				is_private:    ezscm_vars.yes_no.yes,
				s_id:          $(this).data("s_id"),
				time_begin:    moment(time_internal, "HH:mm:ss").format(schedule_options.time_format.value),
				time_internal: time_internal
			};

			show_entry(entry);
		}
		else {
			$(".entry-details").addClass("hidden");
		}

		return false;
	});

	/**
		option add/remove buttons
	**/
	// add
	$("body").on("click", ".option-add", function() {
		option_add($(this).parents(".option-wrapper"));
		ui_init();

		return false;
	});
	// remove
	$("body").on("click", ".option-remove", function() {
		form_changed = true;

		$(this).parents(".option-item").remove();

		return false;
	});

	function option_add(option_wrapper) {
		var clone             = option_wrapper.find(".option-clone");
		var item_last         = option_wrapper.find(".option-item:last");
		var item_last_row     = item_last.data("row");
		var item_last_row_new = parseInt(item_last_row) + 1;

		// generate new input name
		var item_input_new = option_wrapper.data("option_name") + "[" + option_wrapper.data("option_id") + "]";
		if (item_last_row || item_last_row === 0) {
			item_input_new += "[" + item_last_row_new + "]";
		}
		// additional form names
		var inputnames = [];
		if (option_wrapper.data("inputnames")) {
			inputnames = option_wrapper.data("inputnames").split(",");
		}


		var item_new = clone.clone()
			.attr("data-row", item_last_row_new)
			.removeClass("hidden option-clone")
			.find("input").each(function(i, el) {
				var item_input_add = "[]";

				if (typeof inputnames[i] !== "undefined") {
					item_input_add = "[" + inputnames[i] + "]";
				}

				$(this).attr("name", item_input_new + item_input_add);
			}).end()
			.insertAfter(item_last);

		ui_daterange_init();

		return item_new;
	}

	/**
		fill entry visually
	**/
	function fill_entry(e, saved, noSelect) {
		var
			entryData      = saved ? e.data : $.parseJSON(e.data),
			entryDate      = e.date_internal || e.date,
			entryID        = e.e_id,
			entryTimeBegin = e.time_internal || e.time_begin,
			entryTimeEnd   = e.time_end,
			entryClass     = e.private==1 ? "private" : "public";

		// set selected entry to target
		var entry_selected = $(".schedule-entry[data-id='" + entryID + "']");
		if (entry_selected.length > 0) {
			var add_free_class = false;
			if (!entry_selected.hasClass("entry-blocked")) add_free_class = true;

			entry_selected
				.removeAttr("data-action data-id")
				.removeClass("entry-free entry-public entry-private entry-selected")
				.find("span")
				.text("");

			if (add_free_class) entry_selected.addClass("entry-free");
		}

		// add data to new entry
		var select = noSelect ? "" : "entry-selected";
		var target = $(".schedule-entry[data-date='" + entryDate + "'][data-time='" + entryTimeBegin + "']");
		target
			.removeClass("entry-free entry-public entry-private")
			.addClass(select + " entry-" + entryClass)
			.attr("data-action", "get_entry")
			.attr("data-id", entryID);

		if (schedule_options.entry_content_backend.value.length > 0 && entryData[schedule_options.entry_content_backend.value]) {
			target.find("span").text(entryData[schedule_options.entry_content_backend.value]);
		}
	}

	/**
		show single entry
	**/
	function show_entry(entry, saved) {
		$(".entry-details").fadeIn("fast");
		$(".details-info").html("");

		var data_details;
		// modify existing entry
		if (!entry.hasOwnProperty("date_internal")) {
			$("#ezscm-edit_id").val(entry.e_id);

			try {
				data_details = $.parseJSON(entry.data);
			}
			catch (e) {
				data_details = {};
				alert("Unable to load entry details.");
			}

			entry["date_internal"] = entry.date;
			entry["time_internal"] = entry.time_begin;
			// put data into seperate var
			entry["date"]          = moment(entry.date, "YYYY-MM-DD").format(schedule_options.date_format.value);
			entry["time_begin"]    = moment(entry.time_begin, "HH:mm:ss").format(schedule_options.time_format.value);
			entry["private"]       = entry.private;
		}
		// new entry
		else {
			$("#ezscm-edit_id").val(-1);
			entry["date_internal"] = entry.date_internal;
			entry["time_internal"] = entry.time_internal;
			// put data into seperate var
			entry["data"]          = [];
			entry["date"]          = moment(entry.date_internal, "YYYY-MM-DD").format(schedule_options.date_format.value);
			entry["time_begin"]    = moment(entry.time_begin, "HH:mm:ss").format(schedule_options.time_format.value);
			entry["private"]       = entry.private;
		}

		// no entry values, fill in defaults
		if (!data_details || data_details.length < 1) {
			data_details = {};

			$.each(form_elements, function(k, element) {
				// remove all options in brackents, e.g. [required], [email] etc.
				var element_sanitized = element.replace(/\[.+?\]/g, "");

				data_details[element_sanitized] = "";
			});
		}

		// populate fields
		$.each(entry, function(k, v) {
			$("#details-" + k).val(v);
		});

		$.each(form_elements, function(i, v) {
			var heading_sanitized  = v.replace(/\[.+?\]/g, "");
			var data_details_value = data_details[heading_sanitized];

			var input = "<input type='text' name='data[" + heading_sanitized + "]' value='" + data_details_value + "' />";

			if (v.indexOf("[textarea]") > -1) {
				input = "<textarea name='data[" + heading_sanitized + "]'>" + data_details_value + "</textarea>";
			}

			var html = "<p>" + heading_sanitized + "<br>" + input + "</p>";

			$(".details-info").append(html)
		});

		if (saved) {
			fill_entry(entry, saved);
		}
	}

	/**
		show entries list
	**/
	function show_entries(entries, options) {
		var out = "";
		$.each(entries, function(k, entry) {
			out += "<div class='ezscm-entry-single'>";
			out += "	<div class='button ezscm-entry-header'>";

			var header_text = moment(entry.date).format(date_format) + " - " + moment(entry.time_begin, "HH:mm:ss").format("HH:mm");
			out += "		<span>" + header_text + "</span>";
			out += "	</div>";

			out += "	<div class='ezscm-entry-data'>";
			$.each(entry, function(name, entryData) {
				out += "<p>" + name + ": " + entryData + "</p>";
			});

			out += "	</div>";
			out += "</div>";
		});

		$(".ezscm-entries-wrapper").html(out);
	}

	/**
		show schedule entries
	**/
	function show_schedule(s_id, entries, options, week) {
		if (typeof options.start_current_weekday === "undefined") options["start_current_weekday"] = { value: 0 };

		// options
		schedule_options = options;

		// get form elements
		form_elements = options.form_elements.value.replace(/(?:\r\n|\r|\n)/g, "").split(",");
		// set language
		moment.locale(options.lang_dates.value);

		// internal blocked times
		var blocked = [];

		if (!options.time_block) {
			blocked.push([
				options.time_block_start.value,
				options.time_block_end.value
			]);
		}
		else {
			var tmp_blocked_split = options.time_block.value.split(",");
			var tmp_blocked_array = [];
			for (var i in tmp_blocked_split) {
				tmp_blocked_array.push(tmp_blocked_split[i].split("-"));
			}

			blocked.push(tmp_blocked_array);
		}

		var days = options.days_available.value.split(",");

		if (!week) {
			week = moment();
		}

		// set internal block times
		var	block_begin, block_end, block_diff;
		if (blocked.length > 0) {
			block_begin = moment(blocked[0], "HH:mm");
			block_end   = moment(blocked[1], "HH:mm");
		}

		// closed days
		var closed_days = $.parseJSON(options.closed_days.value);

		// calculate time steps
		var
			tmp_begin = moment(options.time_begin.value, "HH:mm"),
			tmp_end   = moment(options.time_end.value, "HH:mm"),
			tmp_steps = moment(options.time_steps.value, "HH:mm");

		var
			begin_hour   = tmp_begin.hour(),
			begin_minute = tmp_begin.minute(),
			end_hour     = tmp_end.hour(),
			end_minute   = tmp_end.minute(),
			step_hour    = tmp_steps.hour(),
			step_minute  = tmp_steps.minute();

		var
			tmp_hours   = (end_hour - begin_hour) * 60,
			tmp_minutes = end_minute - begin_minute;

		var steps = Math.ceil((tmp_hours + tmp_minutes) / (step_hour*60 + step_minute));

		// output
		
		// output
		var out = "<div class='ezscm-schedule'>";
		var out_times = "";

		// hours
		out_times += "<ul class='schedule-day schedule-day-times schedule-times-main'>";
		out_times += "	<li class='schedule-header'></li>";

		var tmp_time = moment({ hours: begin_hour, minute: begin_minute });
		for (var h = 0; h < steps; h++) {
			out_times += "<li class='entry-small'>";
			out_times += "	<span>" + tmp_time.format(options.time_format.value) + "</span>";
			out_times += "</li>";

			tmp_time.add(step_hour, "hours").add(step_minute, "minutes");
		}
		out_times += "</ul>";

		out += out_times;

		// days
		var days_show = parseInt(options.show_weeks_amount_backend.value) * 7;
		for (var day_counter = 0; day_counter < days_show; day_counter++) {
			// start with current weekday
			var day_offset = 0;

			if (options.start_current_weekday.value == 1) {
				day_offset = moment().weekday() - 1;
			}

			var is_first_day = false;
			var loop_day;
			var day_counter_offset = day_counter + day_offset;

			// reset back to day-range (0 to 7)
			if (day_counter_offset >= 7) {
				var loop_day_calc = day_counter_offset - Math.floor(day_counter_offset / 7) * 7;

				if (!days[loop_day_calc]) continue;

				loop_day = parseInt(days[loop_day_calc]) + day_counter_offset - loop_day_calc;
				is_first_day = loop_day_calc == day_offset;
			}
			else {
				if (!days[day_counter_offset]) continue;
				if (days[day_counter_offset] == 0) days[day_counter_offset] = 7;

				loop_day = parseInt(days[day_counter_offset]);
			}

			// add times
			if (is_first_day) {
				out += "</div><div class='ezscm-schedule'>";
				out += out_times;
			}

			// set loop date
			var m = moment(week).startOf("isoweek").isoWeekday(loop_day + day_offset).hour(begin_hour).minute(begin_minute);

			var currentDateInternal = moment(week, "YYYY-MM-DD").startOf("isoWeek").isoWeekday(loop_day);
			var currentDate         = moment(week, "YYYY-MM-DD").startOf("isoweek").isoWeekday(loop_day).format("YYYY-MM-DD");
			var currentDay          = moment(week, "YYYY-MM-DD").startOf("isoweek").isoWeekday(loop_day).format(schedule_options.date_format.value);
			var currentDayName      = moment(week, "YYYY-MM-DD").isoWeekday(loop_day).format("dddd");

			// check if day is closed / blocked
			var closed_day = false;
			for (var ci = 0; ci < closed_days.length; ci++) {
				var closed_day_from = moment(closed_days[ci].from, "YYYY-MM-DD");
				var closed_day_to   = moment(closed_days[ci].to, "YYYY-MM-DD");

				if ((currentDateInternal.isAfter(closed_day_from) && currentDateInternal.isBefore(closed_day_to)) ||
					currentDateInternal.isSame(closed_day_from) || currentDateInternal.isSame(closed_day_to)) {
					closed_day = true;
				}
			}

			out += "<ul class='schedule-day'>";
			out += "	<li class='schedule-header'>" + currentDayName + "<br>" + currentDay + "</li>";

			// loop steps
			for (var s = 0; s < steps; s++) {
				var tmp_class;
				var current_data_time = m.format("HH:mm:ss");

				// if day is closed, just add blocked class
				if (closed_day) {
					tmp_class = "entry-blocked";
				}
				else {
					tmp_class = "entry-free";

					/**
						fill blocked times
					**/
					for (var b = 0; b < blocked.length; b++) {
						var block_begin = moment(currentDate + " " + blocked[b][0]);
						var block_end   = moment(currentDate + " " + blocked[b][1]);

						if ((m.isAfter(block_begin) || m.isSame(block_begin)) && m.isBefore(block_end)) {
							tmp_class = "entry-blocked";
							break;
						}
					}
				}

				out += "<li class='schedule-entry " + tmp_class + "' data-date='" + currentDate + "' data-time='" + current_data_time + "' data-s_id='" + s_id + "'>";
				out += "	<span></span>";
				out += "</li>";			

				m.add(step_hour, "hours").add(step_minute, "minutes");
			}

			out += "</ul>";
		}

		$(".ezscm-schedule-wrapper").html(out);


		/**
			fill public entries
		**/
		for (var e = 0; e < entries.length; e++) {
			fill_entry(entries[e], false, true);
		}

		// show legend + option bar
		$(".ezscm-wrapper, .name-wrapper").removeClass("hidden");
		// assign s_id to option buttons
		$(".ezscm-option-button, .ezscm-option-save").data("id", s_id);
		// shortcode
		$("#ezscm_shortcode").val("[ezscm id='" + s_id + "' /]");
		var schedule_name = $(".ezscm-schedules-list .button-primary .schedule-name").text();
		$("#ezscm_shortcode_name").val("[ezscm name='" + schedule_name + "' /]");

		// next / prev week values
		var week_prev = moment(week).subtract(days_show, "days").format("YYYY-MM-DD");
		var week_next = moment(week).add(days_show, "days").format("YYYY-MM-DD");
		
		$(".ezscm-browse-prev").data("week", week_prev);
		$(".ezscm-browse-next").data("week", week_next);

		show_schedule_settings(options);
	}

	/**
		add new schedule
	**/
	function add_schedule_item(id) {
		$(".ezscm-schedules-list li.clone")
			.clone()
			.attr("data-id", id)
			.removeClass("clone")
			.appendTo(".ezscm-schedules-list")
			.html("<i class='fa fa-fw fa-list-alt'></i> " + id + " - <span class='schedule-name'>New schedule</span>")
			.fadeIn("fast")
			.click();
	}


	/**
		show schedule options
	**/
	function show_schedule_settings(options) {
		$.each(options, function(name, v) {
			var target = "#opt-" + v.o_id;

			switch (v.type) {
    			case "yesno":
    			case "lang":
    				$(target + " option").removeAttr("selected");
    				$(target + " option[value='" + v.value + "']").attr("selected", "selected");
    			break;

    			case "weekdays":
    				$(target).val(v.value);

    				// remove all checked buttons first
    				$(target).siblings(".buttonset").find(".ui-state-active").removeClass("ui-state-active");
    				$(target).siblings(".buttonset").find(":checked").each(function() {
    					$(this).attr("checked", "");
    				});

    				var days = v.value.split(",");
    				if (days.length < 1) return;

    				// add checked buttons
    				for (var i in days) {
    					var target_button = v.name + "_" + days[i];

    					// set input field to checked
    					$("#" + target_button).attr("checked", "checked");
    					// add active state class
    					$("label[for='" + target_button + "']").addClass("ui-state-active");
    				}
    			break;

    			case "datepicker_array":
    				var day_wrapper = $(target);
    				day_wrapper.find(".option-item:not(.option-clone)").remove();

    				// days
    				var days = $.parseJSON(v.value);
    				if (days.length < 1) return;

    				$.each(days, function(i, day) {
    					// add item
    					var item = option_add(day_wrapper);

    					// from
    					$(item).find(".datepicker-from").val(day.from);
    					// to
    					$(item).find(".datepicker-to").val(day.to);
    				});
    			break;

    			case "editor":
    				// visual editor
    				$("#editor_" + v.o_id + "_ifr").contents().find("body").html(nl2br(v.value));
    				// textarea
    				$("#editor_" + v.o_id).val(v.value);
    			break;

    			default:
    				$(target).val(v.value);
    			break;
    		}
		});

		ui_update(options);

		$(".ezscm-wrapper").addClass("hidden");
		$(".ezscm-options-schedule-data").removeClass("hidden");
	}


	function entry_delete(id) {
		$(".entry-selected")
			.removeAttr("data-id")
			.removeAttr("data-action")
			.removeClass("entry-private entry-public entry-selected")
			.addClass("entry-free")
				.find("span").text("");
		$(".entry-details").addClass("hidden");
	}


	/**
		ajax
	**/
	function do_action(el) {
		$(".ezscm-loading").fadeIn("fast");

		var action            = $(el).data("action");
		var id                = $(el).data("id");
		var data              = "action="+action;
		var selected_schedule = $(".ezscm-schedules-list .button-primary");
		var s_id              = $(selected_schedule).data("id");
		
		// get schedule entries / options
		switch (action) {
			// delete entry
			case "entry_delete":
				if (!confirm(ezscm_vars.entry_delete)) {
					return false;
				}

				id = $(".entry-selected").data("id");
			break;

			case "get_entries":
				$(".ezscm-wrapper, .shortcode-wrapper").addClass("hidden");
			break;

			case "get_options":
				$(".ezscm-loading").hide();
				$(".ezscm-options-dialog").dialog("open");
				return false;
			break;

			case "get_schedule":
				$(".ezscm-wrapper, .shortcode-wrapper").addClass("hidden");
				$(".ezscm-browse-prev, .ezscm-browse-next").data("id", id);

				var browse_week = $(el).data("week");
				if (browse_week) {
					data += "&week=" + browse_week;
				}
			break;

			// rename schedule name
			case "rename":
				id = s_id;
				var name = $(selected_schedule).find(".schedule-name").text();
				var new_name = prompt("Schedule name", name);

				if (new_name == null) return false;

				data += "&name=" + encodeURIComponent(new_name);
			break;

			// save single entry
			case "save_entry":
				var check_date = $("#details-date_internal").val();
				var check_time = $("#details-time_internal").val();
				var entry_id   = $(".entry-selected").data("id");

				// check if entry exists
				var entry_exists = $(".schedule-entry[data-date='" + check_date + "'][data-time='" + check_time + "']");
				if (entry_exists.length > 0 && !entry_exists.hasClass("entry-free") && entry_exists.data("id") != entry_id) {
					set_message("Entry for " + check_date + ", " + check_time + " already exists.", "ezscm-error");

					$(".ezscm-loading").fadeOut("fast");
					return false;
				}

				var save_data = $(".entry-details form").serialize();
				data += "&" + save_data;
			break;

			// clear schedule
			case "schedule_clear":
				if (!confirm(ezscm_vars.schedule_clear)) {
					$(".ezscm-loading").fadeOut("fast");
					return false;
				}

				id = s_id;
			break;

			// delete schedule
			case "schedule_del":
				if (!confirm(ezscm_vars.schedule_delete)) {
					$(".ezscm-loading").fadeOut("fast");
					return false;
				}

				id = s_id;
			break;

			case "update_options":
				if (typeof tinyMCE !== "undefined") {
					tinyMCE.triggerSave();
				}

				// buttonset -> single value
				$(".ezscm-settings-type-weekdays").each(function() {
					var day_values = [];

					$(this).siblings(".buttonset").find(":checked").each(function() {
						day_values.push($(this).val());
					});

					$(this).val(day_values.join(","));
				});

				var save_data = $("#form-options").serialize();
				data += "&" + save_data;
			break;
		}

		// auto append id
		if (id)   data += "&id=" + id;
		if (s_id) data += "&s_id=" + s_id;

		// nonce
		data += "&nonce=" + ezscm_nonce;

		$.ajax({
			type: "post",
			url: ajaxurl,
			data: {
				action: "ezscm_backend",
				data: data
			},
			success: function(response) {
				$(".ezscm-loading").fadeOut("fast");

				//console.log(response);

				response = $.parseJSON(response);
				if (!response) return false;

				/**
					error occured
				**/
				if (response.error) {
					set_message(response.success, "ezscm-error");
					return false;
				}

				/**
					success
				**/
				if (response.success) {
					set_message(response.success);
				} 

				/**
					map options
				**/
				if (response.options) {
					var tmp_options = {};

					for (var i in response.options) {
						tmp_options[response.options[i].name] = response.options[i];
					}

					response.options = tmp_options;
				}

				/**
					call functions after ajax request
				**/
				switch (action) {
					case "add":
						add_schedule_item(response.id);
					break;

					case "entry_delete":
						entry_delete(id);
					break;

					case "get_entry":
						show_entry(response);
						$(".entry-details").removeClass("hidden");
					break;

					case "get_entries":
						show_entries(response);
						$(".ezscm-entries-wrapper").removeClass("hidden");
					break;

					case "get_schedule":
						show_schedule(id, response.entries, response.options, browse_week);

						$(".ezscm-wrapper, .shortcode-wrapper").removeClass("hidden");
						$(".entry-details").addClass("hidden");
					break;

					case "rename":
						$(selected_schedule).find(".schedule-name").text(response);
						$("#ezscm_shortcode_name").val("[ezscm name='" + response + "' /]");
					break;

					case "save_entry":
						show_entry(response, true);
					break;

					case "schedule_clear":
						// todo
					break;

					case "schedule_del":
						$(".ezscm-schedules-list .button-primary, .ezscm-wrapper, .entry-details, .shortcode-wrapper").addClass("hidden");
					break;

					case "update_options":
						$(".ezscm-options-dialog").dialog("close");

						show_schedule(s_id, response.entries, response.options, browse_week);
						$(".ezscm-wrapper").removeClass("hidden");
					break;
				}

				$(".ezscm-loading").fadeOut();
			}
		});

		return false;
	}

	function ui_init() {
		// datepicker browse
		$(".ezscm-browse-pick-dummy").datepicker($.extend(true, datepicker_defaults, {
			dateFormat: "YYYY-MM-DD",
			showWeek: true,
		    onSelect: function(date, inst) {
		    	$(".ezscm-browse-next").data("week", moment(date).format("YYYY-MM-DD")).trigger("click");
		    }
		}));
		// show up datepicker when icon was clicked
		$(".ezscm-browse-pick").click(function() {
			$(".ezscm-browse-pick-dummy").datepicker("show");
		});

		// timepicker options
		$(".ezscm-settings-type-time").timepicker({
			"step": 10,
	        "timeFormat": "H:i"
	    });

	    // buttonset
	    $(".buttonset").buttonset();
	}

	// set ui options dynamically
	function ui_update(options) {
		$.datepicker.formatDate = function (format, value) {
		    return moment(value).format(format);
		};

		// set language
		moment.locale(options.lang_dates.value);

		// default datepicker options
		datepicker_defaults = {
			changeMonth: true,
			changeYear: true,
			dateFormat: schedule_options.date_format.value,
			defaultDate: datepicker_momentjs_date(moment(), schedule_options.date_format.value),
			showOtherMonths: true,
			selectOtherMonths: true,
			showWeek: true,
			firstDay: 1
		};

		// datepicker details-date
		$("#details-date").datepicker($.extend(true, datepicker_defaults, {
			onSelect: function(date, inst) {
				var date_converted = moment(date, schedule_options.date_format.value).format(date_format_internal);
				$("#details-date_internal").val(date_converted);
			}
		}));

		// calculate time steps
		var
			tmp_begin = moment(options.time_begin.value, "HH:mm"),
			tmp_end   = moment(options.time_end.value, "HH:mm"),
			tmp_steps = moment(options.time_steps.value, "HH:mm");

		var
			begin_hour   = tmp_begin.hour(),
			begin_minute = tmp_begin.minute(),
			end_hour     = tmp_end.hour(),
			end_minute   = tmp_end.minute(),
			step_hour    = tmp_steps.hour(),
			step_minute  = tmp_steps.minute();

		var
			tmp_hours   = (end_hour - begin_hour) * 60,
			tmp_minutes = end_minute - begin_minute;

		var steps = Math.floor(step_hour*60 + step_minute);

		// timepicker entry details
		$("#details-time_begin").timepicker({
			"minTime": tmp_begin.format("H:mm"),
			"maxTime": tmp_end.format("H:mm"),
			"step": steps,
			timeFormat: function(date) {
				return moment(date).format(schedule_options.time_format.value);
			}
		});

		$("#details-time_begin").on("change", function() {
			var time = moment($(this).val(), schedule_options.time_format.value).format("H:mm:ss");
			$("#details-time_internal").val(time);
		});

		ui_daterange_init();
	}

	function ui_daterange_init() {
		// datepicker range
		$(".datepicker-range-wrapper .option-item:not(.option-clone)").each(function() {
			var wrapper = this;

			var datepicker_from = $(this).find(".datepicker-from");
			datepicker_from.datepicker($.extend(true, datepicker_defaults, {
				dateFormat: "YYYY-MM-DD",
				onSelect: function(date) {
					var minDate = datepicker_momentjs_date(date, "YYYY-MM-DD");

					var date_to_element = $(wrapper).find(".datepicker-to");
					var date_to_val = date_to_element.val();

					date_to_element.datepicker("option", "minDate", minDate);
					// due to datepicker's overwritten function formatDate, the value is cleared after selecting
					date_to_element.val(date_to_val);
				}
			}));

			var datepicker_to = $(this).find(".datepicker-to");
			datepicker_to.datepicker($.extend(true, datepicker_defaults, {
				dateFormat: "YYYY-MM-DD",
				onSelect: function(date) {
					var maxDate = datepicker_momentjs_date(date, "YYYY-MM-DD");

					var date_from_element = $(wrapper).find(".datepicker-from");
					var date_from_val = date_from_element.val();

					date_from_element.datepicker("option", "maxDate", maxDate);
					// due to datepicker's overwritten function formatDate, the value is cleared after selecting
					date_from_element.val(date_from_val);
				}
			}));
		});
	}

	function datepicker_momentjs_date(date, date_format) {
    	return moment(date, date_format).toDate();
	}

	function set_message(text, css_class) {
		if (!css_class) css_class = "ezscm-notification";

		$(".ezscm-message").html(" - <span class='" + css_class + "'>" + text + "</span>").slideDown();

		setTimeout(function() {
			$(".ezscm-message").fadeOut();
		}, 7500);
	}

	function nl2br (str, is_xhtml) {
	    var breakTag = (is_xhtml || typeof is_xhtml === 'undefined') ? '<br />' : '<br>';
	    return (str + '').replace(/([^>\r\n]?)(\r\n|\n\r|\r|\n)/g, '$1' + breakTag + '$2');
	}
};

jQuery(document).ready(function($) {
	ezscm_backend($);
});