<?php

class Ezscm_backend {
	public function __construct() {
		global $wpdb;
		$this->wpdb = $wpdb;

		$this->tables = array(
			"debug"             => "{$this->wpdb->prefix}ezscm_debug",
			"entries"			=> "{$this->wpdb->prefix}ezscm_entries",
			"schedules"			=> "{$this->wpdb->prefix}ezscm_schedules",
			"settings"			=> "{$this->wpdb->prefix}ezscm_settings",
			"settings_schedule"	=> "{$this->wpdb->prefix}ezscm_settings_schedule"
		);
	}

	/**
		debug
	**/
	public function get_debug_log() {
		$res = $this->wpdb->get_results("SELECT * FROM {$this->tables["debug"]}");

		if (count($res) < 1) return "No debug logs found.";

		$logs = array();
		foreach ($res as $log) {
			$logs[] = "{$log->time}: {$log->msg}";
		}

		return implode("\n", $logs);
	}

	public function clear_debug_log() {
		$this->wpdb->query("TRUNCATE TABLE `{$this->tables["debug"]}`");
	}

	public function debug($msg) {
		if (get_option("ezscm_debug_mode", 0) == 0) return;

		$this->wpdb->insert(
			$this->tables["debug"],
			array("msg" => $msg),
			array("%s")
		);
	}


	public function entry_delete($id) {
		$id = (int) $id;

		$res = $this->wpdb->delete(
			$this->tables["entries"],
			array(
				"e_id" => $id,
			),
			array(
				"%d",
			)
		);

		return $this->send_message("success", __("Entry deleted.", "ezscm"));
	}


	public function get_entry($id) {
		$id = (int) $id;

		$res = $this->wpdb->get_row("SELECT * FROM {$this->tables["entries"]} WHERE e_id={$id}");

		return $res;
	}

	public function get_entries($id=-1, $week=null, $limit=-1) {
		$sql_params = array();

		$id = (int) $id;
		$sql_id = "";
		if ($id != -1) {
			$sql_id = "AND s_id={$id}"; 
		}

		$sql_limit = "";
		if ($limit != -1) {
			$limit = (int) $limit;
			$sql_limit = "LIMIT {$limit}";
		}

		if ($week) {
			$options = $this->get_schedule_settings($id, true);
			
			$week_start = $this->last_monday($week);
			//$week_end   = date("Y-m-d", strtotime("$week next Sunday"));

			$date_end_days = ((int) $options["show_weeks_amount_backend"]) * 7;
			$week_end      = date("Y-m-d", strtotime("$week +{$date_end_days} days"));

			$res = $this->wpdb->get_results($this->wpdb->prepare(
				"SELECT * FROM {$this->tables["entries"]} WHERE 1=1 {$sql_id} AND `date` BETWEEN %s AND %s ORDER BY e_id DESC {$sql_limit}",
				$week_start, $week_end
			));
		}
		else {
			$res = $this->wpdb->get_results(
				"SELECT * FROM {$this->tables["entries"]} WHERE 1=1 {$sql_id} ORDER BY e_id DESC {$sql_limit}"
			);
		}
		
		
		return $res;
	}

	public function get_schedules() {
		$res = $this->wpdb->get_results("SELECT * FROM {$this->tables["schedules"]}");

		return $res;
	}

	public function get_schedule_settings($id, $values_only = false, $index_key = "name") {
		if (!$id) return Ezscm_functions::send_message("error", __("No ID", "ezfc"));

		$settings = Ezscm_functions::array_index_key($this->get_settings_fields(true), $index_key);

		// merge values
		$settings_db = Ezscm_functions::array_index_key($this->wpdb->get_results($this->wpdb->prepare("SELECT o_id, value FROM {$this->tables["settings_schedule"]} WHERE s_id=%d", $id), ARRAY_A), "o_id");

		foreach ($settings as &$setting) {
			if (!empty($settings_db[$setting["id"]])) $setting["value"] = maybe_unserialize($settings_db[$setting["id"]]["value"]);
			else if (isset($setting["default"])) {
				$setting["value"] = maybe_unserialize($setting["default"]);
			}

			if ($values_only) {
				$setting = maybe_serialize($setting["value"]);
			}
		}

		return $settings;
	}

	/**
		form settings
	**/
	public function get_settings_fields($flat = false) {
		return Ezscm_settings::get_schedule_options($flat);
	}

	/**
		form default settings
	**/
	public function get_settings($flat = false) {
		$settings = $this->get_settings_fields();

		// merge values
		$settings_db = Ezscm_functions::array_index_key($this->wpdb->get_results("SELECT id, value FROM {$this->tables["settings"]}", ARRAY_A), "id");

		foreach ($settings as $cat => &$settings_cat) {
			foreach ($settings_cat as &$setting) {
				if (!empty($settings_db[$setting["id"]])) {
					$setting["value"] = $settings_db[$setting["id"]]["value"];
				}
			}
		}

		if ($flat) return Ezscm_settings::flatten($settings);

		return $settings;
	}

	public function schedule_rename($id, $name) {
		$res = $this->wpdb->update(
			$this->tables["schedules"],
			array("name" => $name),
			array("s_id" => $id),
			array("%s"),
			array("%d")
		);

		if ($res === false) {
			return $this->send_message("error", $this->wpdb->last_error);
		}

		return $name;
	}


	public function save_entry($save_data) {
		$data_details = json_encode($save_data["data"]);

		$sqldata = array(
			"s_id"       => $save_data["details-s_id"],
			"data"       => $data_details,
			"date"       => $save_data["details-date_internal"],
			"private"    => isset($save_data["details-private"]) ? $save_data["details-private"] : 0,
			"time_begin" => $save_data["details-time_internal"],
			"ip"         => $_SERVER["REMOTE_ADDR"]
		);
		$sqlfields = array("%d", "%s", "%s", "%d", "%s", "%s");

		// add entry
		if ($save_data["ezscm-edit_id"] == -1) {
			$res = $this->wpdb->insert(
				$this->tables["entries"],
				$sqldata,
				$sqlfields
			);
		}
		// update entry
		else {
			$res = $this->wpdb->update(
				$this->tables["entries"],
				$sqldata,
				array("e_id" => $save_data["ezscm-edit_id"]),
				$sqlfields,
				array("%d")
			);
		}

		if ($res === false) {
			return $this->send_message("error", $this->wpdb->last_error);
		}

		// return last inserted id or edited id
		$ret_id = $save_data["ezscm-edit_id"]==-1 ? $this->wpdb->insert_id : $save_data["ezscm-edit_id"];

		return $this->send_message("success", __("Entry updated", "ezscm"), $ret_id);
	}


	public function schedule_add() {
		$res = $this->wpdb->insert(
			$this->tables["schedules"],
			array("name" => "New Schedule"),
			array("%s")
		);

		if ($res === false) {
			return $this->send_message("error", $this->wpdb->last_error);
		}

		$ins_id = $this->wpdb->insert_id;

		$ins = $this->wpdb->query("
			INSERT INTO {$this->tables["settings_schedule"]}
				(s_id, o_id, value)
			SELECT
				{$ins_id}, id, value
			FROM
				{$this->tables["settings"]}
		");

		if ($ins === false) {
			return $this->send_message("error", $this->wpdb->last_error);
		}

		return $this->send_message("success", __("Schedule added", "ezscm"), $ins_id);
	}

	public function schedule_clear($id) {
		$res = $this->wpdb->delete(
			$this->tables["entries"],
			array("s_id" => $id),
			array("%d")
		);

		if ($res === false) {
			return $this->send_message("error", $this->wpdb->last_error);
		}

		return $this->send_message("success", __("Schedule cleared", "ezscm"));
	}

	public function schedule_del($id) {
		$res = $this->wpdb->delete(
			$this->tables["schedules"],
			array("s_id" => $id),
			array("%d")
		);

		if ($res === false) {
			return $this->send_message("error", $this->wpdb->last_error);
		}

		$res_del = $this->wpdb->delete(
			$this->tables["settings_schedule"],
			array("s_id" => $id),
			array("%d")
		);

		if ($res_del === false) {
			return $this->send_message("error", $this->wpdb->last_error);
		}

		$res_entries = $this->wpdb->delete(
			$this->tables["entries"],
			array("s_id" => $id),
			array("%d")
		);

		if ($res_entries === false) {
			return $this->send_message("error", $this->wpdb->last_error);
		}

		return $this->send_message("success", __("Schedule deleted", "ezscm"));
	}

	public function update_settings($settings, $overwrite = 0, $upgrade = 0) {
		if (!$settings) $settings = array();

		foreach ($settings as $id => $value) {
			// option has multiple fields
			if (is_array($value)) {
				$tmpvalue = array();

				// remove 'clone'-items - they are required because the option would have no value otherwise
				foreach ($value as $value_array_key => $value_array_value) {
					if ($value_array_key == -1) continue;
					$tmpvalue[] = $value_array_value;
				}

				$value = json_encode($tmpvalue);
			}
			else {
				$value = stripslashes($value);
			}

			$res = $this->wpdb->replace(
				$this->tables["settings"],
				array(
					"id" => $id,
					"value" => $value
				),
				array(
					"%d",
					"%s"
				)
			);

			if ($res === false) {
				return $this->send_message("error", $this->wpdb->last_error);
			}

			// overwrite settings
			if ($overwrite == 1) {
				$res = $this->wpdb->update(
					$this->tables["settings_schedule"],
					array("value" => $value),
					array("o_id" => $id),
					array("%s"),
					array("%d")
				);
			}

			if ($res === false) {
				return $this->send_message("error", $this->wpdb->last_error);
			}
		}

		if ($upgrade == 1) {
			$this->setup_db();
			$this->upgrade();
		}

		return $this->send_message("success", __("Settings updated", "ezscm"));
	}

	public function update_settings_schedule($settings, $s_id) {
		if (count($settings) < 1) return $this->send_message("success", __("Settings updated", "ezscm"));

		foreach ($settings as $o_id => $value) {
			// option has multiple fields
			if (is_array($value)) {
				$tmpvalue = array();

				// remove 'clone'-items - they are required because the option would have no value otherwise
				foreach ($value as $value_array_key => $value_array_value) {
					if ($value_array_key == -1) continue;
					$tmpvalue[] = $value_array_value;
				}

				$value = json_encode($tmpvalue);
			}
			else {
				$value = stripslashes($value);
			}

			$res = $this->wpdb->replace(
				$this->tables["settings_schedule"],
				array(
					"o_id"  => $o_id,
					"s_id"  => $s_id,
					"value" => $value
				),
				array(
					"%d",
					"%d",
					"%s"
				)
			);

			if ($res === false) {
				return $this->send_message("error", $this->wpdb->last_error);
			}
		}

		return $this->send_message("success", __("Settings updated", "ezscm"));
	}

	public function last_monday($date) {
		if (!is_numeric($date)) {
		    $date = strtotime($date);
		}

		if (date('w', $date) == 1) {
		    return $date;
		}
		else {
		    return strtotime(
		        'last monday',
		         $date
		    );
		}
	}


	public function send_message($type, $msg, $id=0) {
		return array(
			$type 	=> $msg,
			"id"	=> $id
		);
	}

	public function upgrade($force = false) {
		$old_version = get_option("ezscm_version", "1.0");

		// do not perform table changes when old version is current version
		if (version_compare($old_version, EZSCM_VERSION) < 0 || $force) {
			// version specific upgrades
			$query_upgrade = array();

			$query_upgrade[] = "ALTER TABLE `{$this->wpdb->prefix}ezscm_settings` DROP COLUMN `name`";

			// do it already!
			foreach ($query_upgrade as $q) {
				$res = @$this->wpdb->query($q);
			}
		}

		// all upgrades finished, update version
		update_option("ezscm_version", EZSCM_VERSION);
	}

	public function add_missing_schedule_options() {
		// deprecated
	}

	public function setup_db() {
		$query = file_get_contents(EZSCM_PATH . "db.sql");
		if (!$query) {
			die("Error opening file 'db.sql'");
		}

		$query_replaced = str_replace("__PREFIX__", $this->wpdb->prefix, $query);
		$this->execute_multiline_sql($query_replaced);
	}

	private function execute_multiline_sql($sql, $delim=";") {
	    global $wpdb;
	    
	    $sqlParts = $this->split_sql_file($sql, $delim);
	    foreach($sqlParts as $part) {
	        $res = $wpdb->query($part);

	        if ($res === false) {
	        	$wpdb->print_error();
	        	return false;
	        }
	    }

	    return true;
	}

	private function split_sql_file($sql, $delimiter) {
	   // Split up our string into "possible" SQL statements.
	   $tokens = explode($delimiter, $sql);

	   // try to save mem.
	   $sql = "";
	   $output = array();

	   // we don't actually care about the matches preg gives us.
	   $matches = array();

	   // this is faster than calling count($oktens) every time thru the loop.
	   $token_count = count($tokens);
	   for ($i = 0; $i < $token_count; $i++)
	   {
	      // Don't wanna add an empty string as the last thing in the array.
	      if (($i != ($token_count - 1)) || (strlen($tokens[$i] > 0)))
	      {
	         // This is the total number of single quotes in the token.
	         $total_quotes = preg_match_all("/'/", $tokens[$i], $matches);
	         // Counts single quotes that are preceded by an odd number of backslashes,
	         // which means they're escaped quotes.
	         $escaped_quotes = preg_match_all("/(?<!\\\\)(\\\\\\\\)*\\\\'/", $tokens[$i], $matches);

	         $unescaped_quotes = $total_quotes - $escaped_quotes;

	         // If the number of unescaped quotes is even, then the delimiter did NOT occur inside a string literal.
	         if (($unescaped_quotes % 2) == 0)
	         {
	            // It's a complete sql statement.
	            $output[] = $tokens[$i];
	            // save memory.
	            $tokens[$i] = "";
	         }
	         else
	         {
	            // incomplete sql statement. keep adding tokens until we have a complete one.
	            // $temp will hold what we have so far.
	            $temp = $tokens[$i] . $delimiter;
	            // save memory..
	            $tokens[$i] = "";

	            // Do we have a complete statement yet?
	            $complete_stmt = false;

	            for ($j = $i + 1; (!$complete_stmt && ($j < $token_count)); $j++)
	            {
	               // This is the total number of single quotes in the token.
	               $total_quotes = preg_match_all("/'/", $tokens[$j], $matches);
	               // Counts single quotes that are preceded by an odd number of backslashes,
	               // which means they're escaped quotes.
	               $escaped_quotes = preg_match_all("/(?<!\\\\)(\\\\\\\\)*\\\\'/", $tokens[$j], $matches);

	               $unescaped_quotes = $total_quotes - $escaped_quotes;

	               if (($unescaped_quotes % 2) == 1)
	               {
	                  // odd number of unescaped quotes. In combination with the previous incomplete
	                  // statement(s), we now have a complete statement. (2 odds always make an even)
	                  $output[] = $temp . $tokens[$j];

	                  // save memory.
	                  $tokens[$j] = "";
	                  $temp = "";

	                  // exit the loop.
	                  $complete_stmt = true;
	                  // make sure the outer loop continues at the right point.
	                  $i = $j;
	               }
	               else
	               {
	                  // even number of unescaped quotes. We still don't have a complete statement.
	                  // (1 odd and 1 even always make an odd)
	                  $temp .= $tokens[$j] . $delimiter;
	                  // save memory.
	                  $tokens[$j] = "";
	               }

	            } // for..
	         } // else
	      }
	   }

	   return $output;
	}

	/**
		test mail
	**/
	public function send_test_mail($recipient) {
		$subject = __("ez Schedule Manager Test Email", "ezscm");
		$text    = __("This is a test email sent by ez Schedule Manager.", "ezscm");

		$res = wp_mail(
			$recipient,
			$subject,
			$text
		);

		return $res==1 ? __("Mail successfully sent.", "ezscm") : __("Unable to send mails.", "ezscm");
	}
}