<?php

class Ezscm_frontend {
	function __construct() {
		global $wpdb;
		$this->wpdb = $wpdb;

		$this->tables = array(
			"debug"             => "{$this->wpdb->prefix}ezscm_debug",
			"entries"			=> "{$this->wpdb->prefix}ezscm_entries",
			"schedules"			=> "{$this->wpdb->prefix}ezscm_schedules",
			"settings"			=> "{$this->wpdb->prefix}ezscm_settings",
			"settings_schedule"	=> "{$this->wpdb->prefix}ezscm_settings_schedule"
		);
	}

	function debug($msg) {
		if (get_option("ezscm_debug_mode", 0) == 0) return;

		$this->wpdb->insert(
			$this->tables["debug"],
			array("msg" => $msg),
			array("%s")
		);
	}

	function get_entries($id, $week=-1) {
		$id         = (int) $id;
		$options    = $this->get_schedule_settings_values($id);

		$isoweek    = strtotime(date("Y")."W".date("W"));
		$week       = $week==-1 ? date("Y-m-d", $isoweek) : $week;
		$date_start = $week;

		$date_end_days = ((int) $options["show_weeks_amount_backend"]) * 7;
		$date_end      = date("Y-m-d", strtotime("$week +{$date_end_days} days"));

		$sql_range  = "AND date BETWEEN %s AND %s";
		$sql_params = array(
			$id,
			$date_start,
			$date_end
		);

		// add content field to schedule
		if (!empty($options["entry_content_frontend"])) {
			$res = $this->wpdb->get_results($this->wpdb->prepare(
				"SELECT time_begin, date, data FROM {$this->tables["entries"]} WHERE s_id=%d {$sql_range}",
				$sql_params
			));

			if (!empty($options["entry_content_frontend"]) && count($res) > 0) {
				$content_field = $options["entry_content_frontend"];

				foreach ($res as &$entry) {
					$entry_data = json_decode($entry->data);

					// just overwrite the data field if
					if (property_exists($entry_data, $content_field)) {
						$entry->data = $entry_data->$content_field;
					}
				}
			}
		}
		// schedule only
		else {
			$res = $this->wpdb->get_results($this->wpdb->prepare(
				"SELECT time_begin, date FROM {$this->tables["entries"]} WHERE s_id=%d {$sql_range}",
				$sql_params
			));
		}
		
		return $res;
	}

	function get_schedule($id, $name=null) {
		if (!$id && !$name) return $this->send_message("error", __("No id or name found.", "ezscm"));

		if ($id) {
			$res = $this->wpdb->get_row($this->wpdb->prepare(
				"SELECT * FROM {$this->tables["schedules"]} WHERE id=%d",
				$id
			));
		}

		if ($name) {
			$res = $this->wpdb->get_row($this->wpdb->prepare(
				"SELECT * FROM {$this->tables["schedules"]} WHERE name=%s",
				$name
			));
		}

		return $res;
	}


	public function get_schedule_settings($s_id, $remove_options = true, $preview_options = false) {
		if (!$s_id && !$preview_options) return Ezscm_Functions::send_message("error", __("No ID", "ezscm"));

		$settings = Ezscm_Functions::array_index_key(Ezscm_settings::get_schedule_options(true), "id");

		// merge values
		if ($preview_options) {
			$settings_db = json_decode(json_encode($preview_options), true);
		}
		else {
			$settings_db = Ezscm_Functions::array_index_key($this->wpdb->get_results($this->wpdb->prepare("SELECT o_id, value FROM {$this->tables["settings_schedule"]} WHERE s_id=%d", $s_id), ARRAY_A), "o_id");
		}

		foreach ($settings as &$setting) {
			if (isset($settings_db[$setting["id"]])) {
				$setting["value"] = maybe_unserialize($settings_db[$setting["id"]]["value"]);
			}
			else {
				$setting["value"] = empty($setting["default"]) ? "" : $setting["default"];
			}
		}

		return $settings;
	}

	public function get_schedule_settings_values($s_id, $preview_options=false) {
		if ($preview_options) {
			$settings_tmp = $this->get_schedule_settings(null, $preview_options);
		}
		else {
			$settings_tmp = $this->get_schedule_settings($s_id);
		}

		$settings = array();

		foreach ($settings_tmp as &$setting) {
			$settings[$setting["name"]] = $setting["value"];
		}

		return $settings;
	}


	/**
		save public entry
	**/
	function save_entry($save_data) {
		$options = $this->get_schedule_settings_values($save_data["s_id"]);

		if ($options["display_only"] == 1) {
			return $this->send_message("error", __("You are not allowed to submit anything.", "ezscm"));
		}

		// look for existing entry
		$existing = $this->wpdb->get_row($this->wpdb->prepare(
			"SELECT 1 FROM {$this->tables["entries"]} WHERE time_begin='%s' AND date='%s' AND s_id=%d",
			$save_data["time_internal"], $save_data["date_internal"], $save_data["s_id"]
		));

		if ($existing) {
			return $this->send_message("error", __("An entry for the requested time already exists. Someone was probably quicker than you.", "ezscm"));
		}

		// spam protection
		$spam_time = $options["spam_time"];

		$spam = $this->wpdb->get_row($this->wpdb->prepare(
			"SELECT 1 FROM {$this->tables["entries"]} WHERE ip='%s' AND date_submitted>=DATE_ADD(NOW(), INTERVAL -{$spam_time} SECOND)",
			$_SERVER["REMOTE_ADDR"]
		));

		if ($spam) {
			return $this->send_message("error", __("Spam protection: you need to wait {$spam_time} seconds before you can add another entry.", "ezscm"));
		}

		// validate form fields
		$elements      = $this->get_element_fields($options["form_elements"]);
		$elements_name = $this->get_element_fields($options["form_elements"], true);

		$error     = "";
		$user_mail = "";
		$valid     = true;

		if (count($elements > 0)) {
			foreach ($elements as $i => $element) {
				$element_name = $elements_name[$i];

				// invalid form data
				if (!isset($save_data["form"][$element_name])) {
					$valid = false;
					return $this->send_message("error", __("Invalid form data. Please reload the page.", "ezscm"));
				}

				$element_value = $save_data["form"][$element_name];

				// required field
				if (strstr($element, "[required]") && empty($element_value)) {
					$valid = false;
					$error = __("Please fill out all required fields.", "ezscm");
				}

				// email
				if (strstr($element, "[email]")) {
					if (!filter_var($element_value, FILTER_VALIDATE_EMAIL)) {
						$valid = false;
						$error = __("Please enter a valid email address.", "ezscm");
					}
					// found user mail
					else {
						$user_mail = $element_value;
					}
				}
			}
		}

		// invalid form
		if (!$valid) {
			return $this->send_message("error", $error);
		}

		$sql_data = json_encode($save_data["form"]);

		$res = $this->wpdb->insert(
			$this->tables["entries"],
			array(
				"s_id"       => $save_data["s_id"],
				"date"       => $save_data["date_internal"],
				"private"    => 0,
				"time_begin" => $save_data["time_internal"],
				"data"		 => $sql_data,
				"ip"         => $_SERVER["REMOTE_ADDR"]
			),
			array(
				"%d",
				"%s",
				"%d",
				"%s",
				"%s",
				"%s"
			)
		);

		if ($res === false) {
			return $this->send_message("error", __("An error has occured.", "ezscm"));
		}

		$this->send_mails($save_data, $options, $user_mail);

		return $this->send_message("success", __($options["submit_message"], "ezscm"), $this->wpdb->insert_id);
	}

	/**
		send mails (obviously)
	**/
	function send_mails($save_data, $options, $user_mail="") {
		$this->debug("Preparing to send mail(s)...");
		$this->debug("Target email: $user_mail");

		// generate email content from submission
		$mail_output = $this->get_mail_output($save_data, $options);

		// sender name
		$mail_from = "";
		if (!empty($options["email_admin_sender"])) {
			$mail_from = $options["email_admin_sender"];
		}

		// admin mail
		if (!empty($options["email_recipient"])) {
			$mail_admin_headers   = array();
			$mail_admin_headers[] = "Content-type: text/html";
			$mail_admin_headers[] = "From: {$mail_from}";

			if (!empty($user_mail)) {
				$mail_admin_headers[] = "Reply-to: \"{$user_mail}\"";
			}

			$res = wp_mail(
				$options["email_recipient"],
				__($options["email_admin_subject"], "ezscm"),
				$mail_output["text_admin"],
				$mail_admin_headers
			);

			$this->debug("Email delivery to admin: $res");
			$this->debug(var_export($mail_admin_headers, true));
		}
		else {
			$this->debug("No admin email recipient found.");
		}

		// user mail
		if (!empty($user_mail)) {
			$mail_subject = $options["email_subject"];

			// headers
			$mail_headers   = array();
			$mail_headers[] = "Content-type: text/html";
			if (!empty($mail_from)) {
				$mail_headers[] = "From: {$mail_from}";
			}

			$res = wp_mail(
				$user_mail,
				$mail_subject,
				$mail_output["text_user"],
				$mail_headers
			);

			$this->debug("Email delivery to user: $res");
			$this->debug(var_export($mail_headers, true));
		}
		else {
			$this->debug("No user email found.");
		}
	}

	/**
		get email output
	**/
	function get_mail_output($save_data, $options) {
		// form fields
		$form_elements = $this->get_element_fields($options["form_elements"], true);

		// put email text into vars
		$mail_content_replace       = $options["email_text"];
		$mail_admin_content_replace = $options["email_admin_text"];

		// output prefix
		$out_pre = "<html><body>";

		// output suffix
		$out_suf = "</body></html>";

		// output buffer
		$out = array();
		$out[] = "<table>";

		// add protected fields
		$form_elements[] = "callbackdate";
		$form_elements[] = "callbackhour";

		$text_date = empty($options["schedule_text_date"]) ? __("Date", "ezscm") : $options["schedule_text_date"];
		$text_time = empty($options["schedule_text_time"]) ? __("Time", "ezscm") : $options["schedule_text_time"];

		// form fields
		foreach ($form_elements as $i => $label) {
			$input_value = $save_data["form"][$label];

			$label_text = $label;

			// change label for date/time
			if ($label == "callbackdate")      $label_text = $text_date;
			else if ($label == "callbackhour") $label_text = $text_time;

			$out[] = "<tr><td>" . __($label_text, "ezscm") . ":";
			$out[] = "</td><td>" . "{$input_value}";
			$out[] = "</td></tr>";

			// replace template fields, e.g. {{name}}
			$mail_content_replace       = str_ireplace("{{" . $label . "}}", $input_value, $mail_content_replace);
			$mail_admin_content_replace = str_ireplace("{{" . $label . "}}", $input_value, $mail_admin_content_replace);
		}

		$out[] = "</table>";

		// implode content
		$result_content = implode("", $out);

		// replace other values
		$replaces = array(
			"result" => $result_content
		);

		foreach ($replaces as $replace => $replace_value) {
			$mail_content_replace       = str_ireplace("{{" . $replace . "}}", $replace_value, $mail_content_replace);
			$mail_admin_content_replace = str_ireplace("{{" . $replace . "}}", $replace_value, $mail_admin_content_replace);
		}

		// put together email contents for user
		$mail_content  = $out_pre;
		$mail_content .= apply_filters("the_content", $mail_content_replace);
		$mail_content .= $out_suf;

		// put together email contents for admin
		$mail_admin_content  = $out_pre;
		$mail_admin_content .= apply_filters("the_content", $mail_admin_content_replace);
		$mail_admin_content .= $out_suf;

		return array(
			"result"     => $result_content,
			"text_user"  => $mail_content,
			"text_admin" => $mail_admin_content
		);
	}


	/**
		output
	**/
	function get_output($id, $name=null, $theme=null) {
		if (!$id && !$name) return __("No id or name found. Correct syntax: [ezscm id='1' /] or [ezscm name='form-name' /]");

		if ($id) {
			$options = $this->get_schedule_settings_values($id);

			if (!$options) return __("No schedule found (ID: {$id}).", "ezscm");
		}

		if ($name) {
			$schedule = $this->get_schedule(null, $name);

			if (!$schedule) return __("No schedule found (Name: {$name}).", "ezscm");
			
			$id = $schedule->s_id;
		}

		$options       = $this->get_schedule_settings_values($id);
		$form_elements = $this->get_element_fields($options["form_elements"]);

		// override theme by shortcode
		if (empty($theme)) {
			$theme = isset($options["theme"]) ? $options["theme"] : "default";
		}

		// theme css
		$theme_file = EZSCM_PATH . "themes/{$theme}.css";
		$theme_def  = EZSCM_PATH . "themes/default.css";
		if (file_exists($theme_file)) {
			wp_enqueue_style("ezscm-theme-{$theme}", plugins_url("themes/{$theme}.css", __FILE__), array(), EZSCM_VERSION);
		}
		else if (file_exists($theme_def)) {
			wp_enqueue_style("ezscm-theme-default", plugins_url("themes/default.css", __FILE__), array(), EZSCM_VERSION);	
		}

		// check if recaptcha was already included / defined otherwise
		if (!defined("RECAPTCHA_API_SERVER")) {
			require_once(EZSCM_PATH . "lib/recaptcha-php-1.11/recaptchalib.php");
		}
		$publickey = get_option("ezscm_captcha_public");

		// js output
		$form_options_js = json_encode(array(
			"browse_next"     => $options["browse_week_text_next"],
			"browse_previous" => $options["browse_week_text_previous"],
			"date_format"     => $options["date_format"],
			"debug_mode"      => get_option("ezscm_debug_mode", 0),
			"redirect_url"    => trim($options["redirect_url"]),
			"skip_week"       => $options["skip_week"],
			"time_format"     => $options["time_format"]
		));

		// html output
		$html  = "<div class='ezscm ezscm-schedule-{$id} ezscm-theme-{$theme}' id='ezscm-schedule-{$id}' data-id='{$id}' data-vars='{$form_options_js}'>";
		// loading
		$html .= "	<div class='ezscm-loading'>" . __('Loading', 'ezscm') . "</div>";
		// container
		$html .= "	<div class='ezscm-container'>";
		$html .= "		<div class='ezscm-box'>";
		
		// browse weeks
		if ($options['browse_weeks'] == 1) {
			$week_prev = date('Y-m-d', strtotime('last Monday'));
			$week_next = date('Y-m-d', strtotime('next Monday'));

			$html .= "
				<div class='ezscm-browse'>
					<div class='alignleft inline'>
						<button class='btn alignleft ezscm-browse-prev' data-action='get_schedule' data-browseweek='{$week_prev}'>&lt; " . __($options["browse_week_text_previous"], 'ezscm') . "</button>
					</div>
					<div class='alignright inline'>
						<button class='btn alignright ezscm-browse-next' data-action='get_schedule' data-browseweek='{$week_next}'>" . __($options["browse_week_text_next"], 'ezscm') . " &gt;</button>
					</div>
					
					<div class='ezscm-clear'></div>
				</div>
			";
		}

		// schedule
		$html .= "<div class='ezscm-schedule-wrapper'></div>";

		// message
		$html .= "	<div class='ezscm-message'></div>";

		// form
		$form_class = $options["show_form_dialog"] == 1 ? "ezscm-dialog ezscm-theme-{$theme}" : "";
		$html .= "<div class='ezscm-details {$form_class}'>";
		$html .= "	<form class='ezscm-form' name='ezscm-form[{$id}]' action='' data-id='ezscm-form[{$id}]'>";
		$html .= "		<input id='ezscm-schedule-s_id-{$id}' class='s_id' name='data[s_id]' type='hidden' value='{$id}' />";
		$html .= "		<input id='ezscm-schedule-date_internal-{$id}' class='date_internal' name='data[date_internal]' type='hidden' />";
		$html .= "		<input id='ezscm-schedule-time_internal-{$id}' class='time_internal' name='data[time_internal]' type='hidden' />";

		$el_pre = "<div class='ezscm-element'>";
		$el_suf = "</div>";

		$text_date = empty($options["schedule_text_date"]) ? __("Date", "ezscm") : $options["schedule_text_date"];
		$text_time = empty($options["schedule_text_time"]) ? __("Time", "ezscm") : $options["schedule_text_time"];

		// callback date
		$html .= "<div class='ezscm-details-element ezscm-details-element-date'>
			<label>{$text_date}</label><input type='text' id='ezscm-schedule-callbackdate-{$id}' name='data[form][callbackdate]' class='callbackdate' readonly value='' />
		</div>";
		// callback hour
		$html .= "<div class='ezscm-details-element ezscm-details-element-time'>
			<label>{$text_time}</label><input type='text' id='ezscm-schedule-callbackhour-{$id}' name='data[form][callbackhour]' class='callbackhour' readonly value='' />
		</div>";

		foreach ($form_elements as $i => $element) {
			$required = "";
			$required_char = "";
			// required field
			if (stristr($element, "[required]")) {
				$required = "required";
				$required_char = " *";
				$element = str_replace("[required]", "", $element);
			}

			// input field
			$el_text = "<input class='ezscm-element-input' type='text' name='data[form][{$element}]' id='ezscm-form-{$id}-{$element}' {$required} />";

			// email field
			if (stristr($element, "[email]")) {
				$element = str_replace("[email]", "", $element);

				$el_text = "<input class='ezscm-element-email ezscm-element-email' type='email' name='data[form][{$element}]' id='ezscm-form-{$id}-{$element}' {$required} />";
			}
			// textarea
			elseif (stristr($element, "[textarea]")) {
				$element = str_replace("[textarea]", "", $element);

				$el_text = "<textarea class='ezscm-element-textarea' type='text' name='data[form][{$element}]' id='ezscm-form-{$id}-{$element}' {$required}></textarea>";
			}
			// dropdown
			elseif (stristr($element, "[dropdown|")) {
				$tmp_options_string = explode("[dropdown|", $element);
				$tmp_options_pieces = explode("]", $tmp_options_string[1]);
				$tmp_options        = explode(";", $tmp_options_pieces[0]);

				$element_options = array();
				foreach ($tmp_options as $opt) {

					$element_options[] = "<option value='{$opt}'>{$opt}</option>";
				}

				$element = preg_replace("/\[dropdown\|.+?\]/i", "", $element);
				$el_text = "<select class='ezscm-element-dropdown' name='data[form][{$element}]' id='ezscm-form-{$id}-{$element}' {$required}>" . implode("", $element_options) . "</select>";
			}

			$el_label = "<label class='ezscm-label' for='ezscm-form-{$id}-{$element}'>{$element}{$required_char}</label>";

			$html .= $el_pre.$el_label.$el_text.$el_suf;
		}

		// recaptcha
		$html .= "<div class='ezscm-details-submit'>";
		$html .= "    <label>&nbsp;</label>";

		if (!empty($privatekey) && !empty($publickey)) {
			$html .= "<h3>" . __('Confirmation', 'ezscm') . "</h3>";
			echo recaptcha_get_html($publickey);
		}

		// submit
		$html .= "    <input class='ezscm-submit {$options["submit_button_css"]}' type='submit' value='" . __($options["submit_button_text"], 'ezscm') . "' />";
		$html .= "</div>";

		// required char
		$html .= "		<div class='ezscm-required-notification'><span class='ezscm-required-char'>*</span> " . __(get_option("ezscm_required_text", "Required"), "ezscm") . "</div>";

		$html .= "	</form>";
		$html .= "</div>";

		// success text
		if (isset($options["success_text"])) {
			$html .= "<div class='ezscm-success-text'>{$options["success_text"]}</div>";
		}

		$html .= "</div></div></div>";

		return $html;
	}

	public function get_element_fields($text, $replace = false) {
		$return_array = explode(",", str_replace(array("\r", "\n"), "", $text));

		if ($replace) {
			foreach ($return_array as &$v) {
				// remove brackets
				$v = preg_replace("/\[.+?\]/i", "", $v);
				// remove \r\n
				$v = trim(preg_replace("/\s\s+/", " ", $v));
			}
		}

		return $return_array;
	}

	function send_message($type, $msg, $id=0) {
		return json_encode(array(
			$type 	=> $msg,
			"id"	=> $id
		));
	}
}

/**
	DateTime wrapper class for older php versions
**/
class EZSCM_DateTime extends DateTime { 
	public static function createFromFormat($format, $time, $timezone = null) { 
		if(!$timezone) $timezone = new DateTimeZone(date_default_timezone_get()); 

		if (version_compare(phpversion(), "5.2.17") > 0) {
			return parent::createFromFormat($format, $time, $timezone); 
		} 

		return new DateTime(date($format, strtotime($time)), $timezone); 
	}
}