<?php

abstract class Ezscm_settings {
	/**
		global settings
	**/
	static function get_global_settings($flat = false) {
		$settings = array(
			"Customization" => array(
				"override_day_names" => array("description" => __("Override day names", "ezscm"), "description_long" => __("The plugin uses default day names. If you want to use your own day names, you can enable this option and change the names in the 'Day names' option.", "ezscm"), "type" => "yesno"),
				"day_names" => array("description" => __("Day names", "ezscm"), "description_long" => __("Day names starting from Monday to Sunday, separated by comma.", "ezscm"), "default" => __("Monday,Tuesday,Wednesday,Thursday,Friday,Saturday,Sunday", "ezscm"), "type" => "input"),
				"custom_css"    => array("description" => __("Custom CSS", "ezscm"), "description_long" => __("Add your custom styles here.", "ezscm"), "type" => "textarea"),
				"required_text" => array("description" => __("Required text", "ezscm"), "description_long" => "", "type" => "input", "default" => "Required")
			),

			"Other" => array(
				"debug_mode"          => array("description" => __("Enable debug mode", "ezscm"), "description_long" => __("This will log certain messages for debugging purposes. You should not enable this in production.", "ezscm"), "type" => "yesno"),
				"uninstall_keep_data" => array("description" => __("Keep data after uninstall", "ezscm"), "description_long" => __("The plugin will keep all plugin-related data in the databse when uninstalling. Only select 'Yes' if you want to upgrade the script.", "ezscm"), "type" => "yesno")
			)
		);

		// get values
		foreach ($settings as $cat => &$settings_cat) {
			foreach ($settings_cat as $name => &$setting) {
				$default = isset($setting["default"]) ? $setting["default"] : "";

				$setting["value"] = get_option("ezscm_{$name}", $default);
			}
		}

		if ($flat) {
			$settings = self::flatten($settings);
		}

		return $settings;
	}

	/**
		update global settings
	**/
	public static function update_global_settings($submitted_values) {
		$settings = self::get_global_settings(true);

		// css array builder
		$css_builder = new EZ_CSS_Builder(".ezscm-wrapper");

		foreach ($settings as $setting_key => $setting) {
			if (!isset($submitted_values[$setting_key])) continue;

			// get post value
			$value = $submitted_values[$setting_key];

			if (is_array($value)) {
				$value = serialize($value);
			}
			else {
				$value = stripslashes($value);
			}

			// update wp option
			update_option("ezscm_{$setting_key}", $value);

			// check for css
			if (!empty($setting["css"]) && !empty($value)) {
				$css_builder->add_css($setting["css"], $value);
			}
		}

		// build css output
		$css_output = $css_builder->get_output();
		update_option("ezscm_css_custom_styling", $css_output);
	}

	/**
		schedule options
	**/
	static function get_schedule_options($flat = false) {
		$settings = array(
			"Schedule" => array(
				"days_available" => array(
					"id" => 1,
					"name" => "days_available",
					"default" => '1,2,3,4,5',
					"description" => __('Available days in a week', "ezscm"),
					"description_long" => __('Selected days will be shown.', "ezscm"),
					"type" => "weekdays"
				),
				"time_begin" => array(
					"id" => 2,
					"name" => "time_begin",
					"default" => '08:00',
					"description" => __('Begin time', "ezscm"),
					"description_long" => __('Earliest entry time.', "ezscm"),
					"type" => "time"
				),
				"time_end" => array(
					"id" => 3,
					"name" => "time_end",
					"default" => '16:00',
					"description" => __('End time', "ezscm"),
					"description_long" => __('Latest entry time. Note: guests cannot add entries after this exact time.', "ezscm"),
					"type" => "time"
				),
				"time_steps" => array(
					"id" => 4,
					"name" => "time_steps",
					"default" => '01:00',
					"description" => __('Entry duration', "ezscm"),
					"description_long" => __('Duration of a single entry. <b>Note: changing this value can hide certain entries due to different start times.</b>', "ezscm"),
					"type" => "time"
				),
				"time_block_start" => array(
					"id" => 5,
					"name" => "time_block_start",
					"default" => '12:00',
					"description" => __('Blocked start time', "ezscm"),
					"description_long" => __('Guests cannot add entries during blocked start- and end time.', "ezscm"),
					"type" => "time"
				),
				"time_block_end" => array(
					"id" => 6,
					"name" => "time_block_end",
					"default" => '13:00',
					"description" => __('Blocked end time', "ezscm"),
					"description_long" => __('Guests cannot add entries during blocked start- and end time.', "ezscm"),
					"type" => "time"
				),
				"show_dates" => array(
					"id" => 8,
					"name" => "show_dates",
					"default" => '1',
					"description" => __('Show dates', "ezscm"),
					"description_long" => __('Show dates in frontend.', "ezscm"),
					"type" => "yesno"
				),
				"show_days" => array(
					"id" => 9,
					"name" => "show_days",
					"default" => '1',
					"description" => __('Show days', "ezscm"),
					"description_long" => __('Show weekday names in frontend.', "ezscm"),
					"type" => "yesno"
				),
				"days_per_row" => array(
					"id" => 10,
					"name" => "days_per_row",
					"default" => '7',
					"description" => __('Days per row', "ezscm"),
					"description_long" => __('Number of days in a row. Depending on the theme, the layout could break. Try a lower number instead.', "ezscm"),
					"type" => "input"
				),
				"browse_weeks" => array(
					"id" => 11,
					"name" => "browse_weeks",
					"default" => '1',
					"description" => __('Browse weeks', "ezscm"),
					"description_long" => __('Guests are able to browse future weeks.', "ezscm"),
					"type" => "yesno"
				),
				"display_only" => array(
					"id" => 12,
					"name" => "display_only",
					"default" => '0',
					"description" => __('Display schedule only', "ezscm"),
					"description_long" => __('Only display the schedule. Guests cannot submit anything.', "ezscm"),
					"type" => "yesno"
				),
				"date_format" => array(
					"id" => 14,
					"name" => "date_format",
					"default" => 'MM-DD-YYYY',
					"description" => __('Date format', "ezscm"),
					"description_long" => __('default: \'MM-DD-YYYY\', see <a href=\'http://momentjs.com/docs/#/parsing/string-format/\' target=\'_blank\'>momentjs.com</a>', "ezscm"),
					"type" => "input"
				),
				"entry_time_ahead" => array(
					"id" => 16,
					"name" => "entry_time_ahead",
					"default" => '24',
					"description" => __('Minimum hours ahead', "ezscm"),
					"description_long" => __('Guests can only submit x hours ahead minimum. Default: 24', "ezscm"),
					"type" => "input"
				),
				"closed_days" => array(
					"id" => 24,
					"name" => "closed_days",
					"default" => '[{"from":"","to":""}]',
					"description" => __('Closed days', "ezscm"),
					"description_long" => __('Timeframes of closed days (for holidays or single days)', "ezscm"),
					"type" => "datepicker_array"
				),
				"time_format" => array(
					"id" => 25,
					"name" => "time_format",
					"default" => 'HH:mm',
					"description" => __('Time format', "ezscm"),
					"description_long" => __('Time format, see <a href="http://momentjs.com/docs/#/displaying/format/" target="_blank">momentjs.com</a> for time formats.', "ezscm"),
					"type" => "input"
				),
				"entry_content_backend" => array(
					"id" => 27,
					"name" => "entry_content_backend",
					"default" => 'Name',
					"description" => __('Entry content (backend)', "ezscm"),
					"description_long" => __('Entry data will be displayed directly in the private schedule.<br>This value must be equal to one element from the option "Form elements", e.g. "Name", "Email" etc.<br>Leave blank for no content.', "ezscm"),
					"type" => "input"
				),
				"entry_content_frontend" => array(
					"id" => 28,
					"name" => "entry_content_frontend",
					"default" => '',
					"description" => __('Entry content (frontend)', "ezscm"),
					"description_long" => __('Entry data will be displayed directly in the public schedule.<br>This value must be equal to one element from the option "Form elements", e.g. "Name", "Email" etc.<br>Leave blank for no content.', "ezscm"),
					"type" => "input"
				),
				"show_weeks_amount_backend" => array(
					"id" => 31,
					"name" => "show_weeks_amount_backend",
					"default" => '1',
					"description" => __('Amount of weeks to be shown in backend', "ezscm"),
					"description_long" => __('', "ezscm"),
					"type" => "input"
				),
				"show_weeks_amount_frontend" => array(
					"id" => 32,
					"name" => "show_weeks_amount_frontend",
					"default" => '1',
					"description" => __('Amount of weeks to be shown in frontend', "ezscm"),
					"description_long" => __('', "ezscm"),
					"type" => "input"
				),
				"browse_week_text_previous" => array(
					"id" => 33,
					"name" => "browse_week_text_previous",
					"default" => 'Previous week',
					"description" => __('Text on button when browsing the previous week', "ezscm"),
					"description_long" => __('', "ezscm"),
					"type" => "input"
				),
				"browse_week_text_next" => array(
					"id" => 34,
					"name" => "browse_week_text_next",
					"default" => 'Next week',
					"description" => __('Text on button when browsing the next week', "ezscm"),
					"description_long" => __('', "ezscm"),
					"type" => "input"
				),
				"skip_week" => array(
					"id" => 38,
					"name" => "skip_week",
					"default" => '0',
					"description" => __('Skip to free entry', "ezscm"),
					"description_long" => __('If no free entries are available in a week, skip to the next week. This will only be checked once.', "ezscm"),
					"type" => "yesno"
				),
				"start_current_weekday" => array(
					"id" => 39,
					"name" => "start_current_weekday",
					"default" => '0',
					"description" => __('Start with current weekday', "ezscm"),
					"description_long" => __('Start with the current weekday if this option is enabled. Otherwise, the week will start with Monday.', "ezscm"),
					"type" => "yesno"
				)
			),
			"Email" => array(
				"email_recipient" => array(
					"id" => 7,
					"name" => "email_recipient",
					"default" => '',
					"description" => __('Email recipient', "ezscm"),
					"description_long" => __('Notifications will be sent to this email. Leave blank for no notifications.', "ezscm"),
					"type" => "input"
				),
				"email_admin_sender" => array(
					"id" => 19,
					"name" => "email_admin_sender",
					"default" => '',
					"description" => __('Sender name', "ezscm"),
					"description_long" => __('Sender name in emails. Use this syntax: Sendername &lt;sender@mail.com&gt;', "ezscm"),
					"type" => "input"
				),
				"email_admin_subject" => array(
					"id" => 20,
					"name" => "email_admin_subject",
					"default" => 'New submission',
					"description" => __('Admin email subject', "ezscm"),
					"description_long" => __('', "ezscm"),
					"type" => "input"
				),
				"email_admin_text" => array(
					"id" => 21,
					"name" => "email_admin_text",
					"default" => '<p>You have received a new submission:</p><p>{{result}}</p>',
					"description" => __('Admin email text', "ezscm"),
					"description_long" => __('Email text sent to the admin. Use {{result}} to display form values', "ezscm"),
					"type" => "editor"
				),
				"email_subject" => array(
					"id" => 22,
					"name" => "email_subject",
					"default" => 'Your submission',
					"description" => __('Email subject', "ezscm"),
					"description_long" => __('', "ezscm"),
					"type" => "input"
				),
				"email_text" => array(
					"id" => 23,
					"name" => "email_text",
					"default" => '<p>Thank you for your submission, we will contact you soon!</p><p>{{result}}</p>',
					"description" => __('Email text', "ezscm"),
					"description_long" => __('Email text sent to the user. Use {{result}} to display form values', "ezscm"),
					"type" => "editor"
				)
			),
			"Form" => array(
				"lang_dates" => array(
					"id" => 13,
					"name" => "lang_dates",
					"default" => 'en',
					"description" => __('Date language', "ezscm"),
					"description_long" => __('Predefined translations for frontend dates. Default: EN', "ezscm"),
					"type" => "lang"
				),
				"spam_time" => array(
					"id" => 15,
					"name" => "spam_time",
					"default" => '60',
					"description" => __('Spam protection in seconds', "ezscm"),
					"description_long" => __('Every x seconds, a user (identified by IP address) can add an entry. Default: 60.', "ezscm"),
					"type" => "input"
				),
				"submit_message" => array(
					"id" => 17,
					"name" => "submit_message",
					"default" => 'Thank you for your submission.',
					"description" => __('Submission message', "ezscm"),
					"description_long" => __('Frontend message after successful submission.', "ezscm"),
					"type" => "input"
				),
				"form_elements" => array(
					"id" => 18,
					"name" => "form_elements",
					"default" => 'Name[required],Email[required][email],Message[required][textarea]',
					"description" => __('Form elements', "ezscm"),
					"description_long" => __('Form elements, separated by comma.<br>Add [required] for required fields.<br>Add [textarea] for large fields (e.g. message field).<br>Add [email] for email field.<br>Default: Name[required],Email[required],Message[required][textarea]', "ezscm"),
					"type" => "textarea"
				),
				"redirect_url" => array(
					"id" => 26,
					"name" => "redirect_url",
					"default" => '',
					"description" => __('Redirect URL', "ezscm"),
					"description_long" => __('Redirect users to this URL upon form submission. Note: URL must start with http://', "ezscm"),
					"type" => "input"
				),
				"submit_button_text" => array(
					"id" => 29,
					"name" => "submit_button_text",
					"default" => 'Submit',
					"description" => __('Submit button text', "ezscm"),
					"description_long" => __('', "ezscm"),
					"type" => "input"
				),
				"submit_button_css" => array(
					"id" => 30,
					"name" => "submit_button_css",
					"default" => '',
					"description" => __('Submit button CSS class', "ezscm"),
					"description_long" => __('', "ezscm"),
					"type" => "input"
				),
				"theme" => array(
					"id" => 35,
					"name" => "theme",
					"default" => 'default',
					"description" => __('Theme', "ezscm"),
					"description_long" => __('', "ezscm"),
					"type" => "themes"
				),
				"schedule_text_date" => array(
					"id" => 36,
					"name" => "schedule_text_date",
					"default" => 'Date',
					"description" => __('Date text', "ezscm"),
					"description_long" => __('', "ezscm"),
					"type" => "input"
				),
				"schedule_text_time" => array(
					"id" => 37,
					"name" => "schedule_text_time",
					"default" => 'Time',
					"description" => __('Time text', "ezscm"),
					"description_long" => __('', "ezscm"),
					"type" => "input"
				),
				"show_form_dialog" => array(
					"id" => 40,
					"name" => "show_form_dialog",
					"default" => '0',
					"description" => __('Show form in dialog', "ezscm"),
					"description_long" => __('', "ezscm"),
					"type" => "yesno"
				)
			)
		);

		if ($flat) {
			$settings = self::flatten($settings);
		}

		return $settings;
	}

	static function flatten($settings) {
		$settings_flat = array();

		foreach ($settings as $cat => $settings_cat) {
			foreach ($settings_cat as $name => $setting) {
				$tmp_id = "";
				
				if (is_array($setting)) {
					if (!empty($setting["id"])) $tmp_id = $setting["id"];
					else if (!empty($setting["name"])) $tmp_id = $setting["name"];
					else $tmp_id = $name;
				}
				else if (is_object($setting)) {
					if (!empty($setting->id)) $tmp_id = $setting->id;
					else if (!empty($setting->name)) $tmp_id = $setting->name;
					else $tmp_id = $name;
				}

				$settings_flat[$tmp_id] = $setting;
			}
		}

		return $settings_flat;
	}

	// wrapper for deprecated extensions / customizations
	public static function array_index_key($array, $key) {
		return ezscm_Functions::array_index_key($array, $key);
	}
}