<?php

defined( 'ABSPATH' ) OR exit;

global $wpdb;

require_once(EZSCM_PATH . "class.ezscm_backend.php");
$ezscm = new Ezscm_backend();

// clear logs
if (isset($_REQUEST["clear_logs"]) && $_REQUEST["clear_logs"] == 1) {
	$ezscm->clear_debug_log();
}

// send test mail
if (isset($_REQUEST["send_test_mail"]) && $_REQUEST["send_test_mail"] == 1 &&
	isset($_REQUEST["send_test_mail_recipient"]) && !empty($_REQUEST["send_test_mail_recipient"])) {
	$message = $ezscm->send_test_mail($_REQUEST["send_test_mail_recipient"]);
}

// vars
$debug_log = $ezscm->get_debug_log();
$install_file = EZSCM_PATH . "db.sql";

$icons = array(
	"good" => "<i class='fa fa-check'></i>",
	"bad"  => "<i class='fa fa-times'></i>"
);

$debug_vars = array(
	"php_version" => array(
		"value"    => PHP_VERSION,
		"required" => "5.4.0",
		"error"    => sprintf(__("PHP version %s or greater is recommended.", "ezscm"), "5.4.0"),
		"is_version" => true
	),
	"wp_version" => array(
		"value"    => get_bloginfo("version"),
		"required" => version_compare(get_bloginfo("version"), "4.1"),
		"error"    => __("WP version 4.1 or greater is recommended.", "ezscm")
	),
	"memory_limit" => array(
		"value"    => @ini_get('memory_limit'),
		"required" => 128,
		"error"    => __("The PHP memory limit is too low. Please contact your webhost to increase the memory limit.", "ezscm")
	),
	"Install file" => array(
		"value"    => file_exists($install_file) && is_readable($install_file),
		"required" => true,
		"is_bool"  => true,
		"error"    => sprintf(__("Please make sure the following file exists and is readable: %s", "ezscm"), $install_file)
	),
	"DB Table schedules" => array(
		"value"    => $wpdb->query("SHOW TABLES LIKE '{$ezscm->tables["schedules"]}'") == 1,
		"required" => true,
		"is_bool"  => true,
		"error"    => __("Plugin was not installed correctly. Please reinstall the plugin.", "ezscm")
	),
	"DB Table settings" => array(
		"value"    => $wpdb->query("SHOW TABLES LIKE '{$ezscm->tables["settings"]}'") == 1,
		"required" => true,
		"is_bool"  => true,
		"error"    => __("Plugin was not installed correctly. Please reinstall the plugin.", "ezscm")
	),
	"upload_max_filesize"   => @ini_get('upload_max_filesize'),
	"post_max_size"         => @ini_get('post_max_size')
);

// generate help table
$help_table = array();
$help_table[] = "<table>";
foreach ($debug_vars as $key => $var) {
	$help_table[] = "<tr><td>";
	$help_table[] = $key;
	$help_table[] = "</td><td>";

	if (is_array($var)) {
		$option_success = false;
		$tmp_out = "";

		if (!empty($var["is_bool"])) {
			$option_success = (bool) $var["value"] == $var["required"];
		}
		else if (!empty($var["version"])) {
			$option_success = version_compare($var["value"], $var["required"]) >= 0;
		}
		else {
			$option_success = (float) $var["value"] >= $var["required"];
			$tmp_out = $var["value"];
		}

		$icon      = $option_success ? $icons["good"] : $icons["bad"];
		$css_class = $option_success ? "ezscm-color-success" : "ezscm-color-error";

		if (!$option_success) {
			if (!empty($var["message"])) $var["error"] = $var["message"];

			$tmp_out .= "<br><strong>{$var["error"]}</strong>";
		}
		else if (!empty($var["message"])) {
			$tmp_out .= "<br>{$var["message"]}";
		}

		$help_table[] = "<span class='{$css_class}'>{$icon} {$tmp_out}</span>";
	}
	else {
		$help_table[] = $var;
	}

	$help_table[] = "</td></tr>";
}
$help_table[] = "</table>";
$help_table_output = implode("", $help_table);

?>

<div class="ezscm wrap">
	<div class="container-fluid">
		<div class="col-lg-12">
			<h2><?php echo __("Help / debug", "ezscm"); ?> - ez Schedule Manager v<?php echo ezscm_get_version(); ?></h2> 
			<p>
				<a class="button button-primary" href="http://mials.de/mials/ezscm/documentation/" target="_blank">Open documentation site</a>
			</p>

			<p>
				If you have found any bugs, please report them to <a href="mailto:support@ezplugins.de">support@ezplugins.de</a>. Thank you!
			</p>
		</div>

		<?php if (!empty($message)) { ?>
			<div class="col-lg-12">
				<div id="message" class="updated"><?php echo $message; ?></div>
			</div>
		<?php } ?>

		<div class="col-lg-6">
			<h3>Debug log</h3>
			<textarea class="ezscm-settings-type-textarea" style="height: 400px; width: 100%;"><?php echo $debug_log; ?></textarea>

			<form action="" method="POST">
				<input type="hidden" value="1" name="clear_logs" />
				<input type="submit" value="Clear logs" class="button button-primary" />
			</form>
		</div>

		<div class="col-lg-3">
			<h3>Environment Vars</h3>

			<?php echo $help_table_output; ?>

			<h3>Tests</h3>

			<div>
				<strong>Emails</strong>
				<form action="" method="POST">
					<input type="hidden" value="1" name="send_test_mail" />
					<input type="text" value="" name="send_test_mail_recipient" placeholder="your@email.com" />
					<br />
					<input type="submit" value="<?php echo __("Send test mail", "ezscm"); ?>" class="button" />
				</form>
			</div>
		</div>

		<div class="col-lg-3">
			<a class="twitter-timeline" href="https://twitter.com/ezPlugins" data-widget-id="575319170478383104">Tweets by @ezPlugins</a>
			<script>!function(d,s,id){var js,fjs=d.getElementsByTagName(s)[0],p=/^http:/.test(d.location)?'http':'https';if(!d.getElementById(id)){js=d.createElement(s);js.id=id;js.src=p+"://platform.twitter.com/widgets.js";fjs.parentNode.insertBefore(js,fjs);}}(document,"script","twitter-wjs");</script>
		</div>
	</div>
</div>
