<?php

defined( 'ABSPATH' ) OR exit;

require_once(EZSCM_PATH . "class.ezscm_functions.php");
require_once(EZSCM_PATH . "class.ezscm_backend.php");
$ezscm = new Ezscm_backend();

// global settings
$settings = Ezscm_settings::get_global_settings();
$settings_raw = Ezscm_settings::get_global_settings(true);

if (isset($_POST["ezscm-reset"])) {
	$keep_data_option = get_option("ezscm_uninstall_keep_data", 0);
	update_option("ezscm_uninstall_keep_data", 0);

	ezscm_uninstall();
	ezscm_register();

	update_option("ezscm_uninstall_keep_data", $keep_data_option);
	$_POST = array();
}

if (isset($_POST["submit"])) {
	$manual_update = isset($_POST["ezscm-manual-update"]) ? $_POST["ezscm-manual-update"] : 0;

	// manual update
	if (!empty($_POST["ezscm-manual-update"])) {
		$ezscm->setup_db();
		$ezscm->upgrade(true);
	}

	// update global options
	Ezscm_settings::update_global_settings($_POST["opt"]);
	// get global settings again
	$settings = Ezscm_settings::get_global_settings();
	$settings_raw = Ezscm_settings::get_global_settings(true);

	$updated = 1;
}

// categorize settings
$settings_cat = array();
foreach ($settings as $cat => $s) {
	$settings_cat[$cat] = $s;
}

?>

<div class="ezscm wrap">
	<?php echo "<h2>" . __("Global settings", "ezscm") . " - v" . EZSCM_VERSION . "</h2>"; ?>

	<?php if (isset($updated)) { ?>
		<div id="message" class="updated"><?php echo __("Settings saved.", "ezscm"); ?></div>
	<?php } ?>

	<form method="POST" name="ezscm-form" class="ezscm-form" action="">
		<div id="ezscm-form-options">
			<div id="tabs">
				<ul>
					<?php
					$tabs = array_keys($settings_cat);

					foreach ($tabs as $i => $cat) {
						echo "<li><a href='#tab-{$i}'>{$cat}</a></li>";
					}
					?>
				</ul>

			    <?php

			    $tab_i = 0;
			    foreach ($settings_cat as $cat_name => $cat) {
			    	?>

					<div id="tab-<?php echo $tab_i; ?>">
						<?php
						echo Ezscm_functions::get_settings_table($cat, "opt", "opt");
						?>
					</div>

					<?php

					$tab_i++;
				}
				?>
			</div>
		</div>

		<table class="form-table" style="margin-top: 1em;">
			<!-- manual update -->
			<tr>
				<th scope='row'>
					<label for="ezscm-manual-update"><?php echo __("Manual update", "ezscm"); ?></label>
		    	</th>
		    	<td>
		    		<input type="checkbox" name="ezscm-manual-update" id="ezscm-manual-update" value="1" /><br>
		    		<p class="description"><?php echo __("Checking this option will perform certain database changes. Check this if you recently updated the script as this will perform necessary changes. <strong>Please make sure to backup your form data before!", "ezscm"); ?></strong></p>
		    		<p class="description"><?php echo __("This will overwrite all default settings but not form options.", "ezscm"); ?></p>
		    	</td>
		    </tr>

			<!-- reset -->
			<tr>
				<th scope='row'>
					<label for="ezscm-reset"><?php echo __("Reset", "ezscm"); ?></label>
		    	</th>
		    	<td>
		    		<input type="checkbox" name="ezscm-reset" id="ezscm-reset" value="1" /><br>
		    		<p class="description"><?php echo __("Complete reset of this plugin. <strong>This will reset all existing data. Use with caution.</strong>", "ezscm"); ?></p>
		    	</td>
		    </tr>
		</table>

		<!-- save -->
		<p class="submit"><input type="submit" name="submit" id="submit" class="button button-primary" value="<?php echo __("Save", "ezscm"); ?>" /></p>
	</form>
</div>

<script>
jQuery(function($) {
	$(".ezscm-form").on("submit", function() {
		// confirmation
		if ($("#ezscm-overwrite").prop("checked")) {
			if (!confirm("Really overwrite all schedule settings?")) return false;
		}

		// buttonset -> single value
		$(".ezscm-settings-type-weekdays").each(function() {
			var day_values = [];

			$(this).siblings(".buttonset").find(":checked").each(function() {
				day_values.push($(this).val());
			});

			$(this).val(day_values.join(","));
		});
	});

	$("#tabs").tabs();
});
</script>