<?php
/*
Plugin Name: ez Schedule Manager
Plugin URI: http://www.mials.de/mials/ezscm/
Description: ez Schedule Manager is a WordPress plugin which allows you to manage incoming schedule requests. In case you offer contact requests or simple booking on specific dates or times, this is the right plugin for you.
Version: 2.3.0
Author: Michael Schuppenies
Author URI: http://www.ezplugins.de/
*/

defined( 'ABSPATH' ) OR exit;

if (defined("EZSCM_VERSION")) return;

/**
	setup
**/
define("EZSCM_VERSION", "2.3.0");
define("EZSCM_PATH", plugin_dir_path(__FILE__));
define("EZSCM_SLUG", plugin_basename(__FILE__));
define("EZSCM_URL", plugin_dir_url(__FILE__));
define("EZSCM_UPDATE_URL", "http://www.ezplugins.de/licensing/versions.php?slug=ezscm");

// ez functions
require_once(EZSCM_PATH . "class.ezscm_functions.php");

// multisite
if ( ! function_exists( 'is_plugin_active_for_network' ) ) {
    require_once( ABSPATH . '/wp-admin/includes/plugin.php' );
}

function ezscm_get_version() {
	return EZSCM_VERSION;
}

/**
	setup
**/
// register plugin
function ezscm_register() {
	require_once(EZSCM_PATH . "ezscm-register.php");
}
// uninstall plugin
function ezscm_uninstall() {
	require_once(EZSCM_PATH . "ezscm-uninstall.php");
}

register_activation_hook(__FILE__, "ezscm_register");
register_uninstall_hook(__FILE__, "ezscm_uninstall");


class EZSCM {
	/**
		init plugin
	**/
	static function init() {
		// setup pages
		add_action("admin_menu", array(__CLASS__, "admin_menu"));
		// check for updates
		add_action("init", array(__CLASS__, "check_updates"));

		// load backend scripts / styles
		add_action("admin_enqueue_scripts", array(__CLASS__, "load_scripts"));

		// settings page
		$ezscm_plugin_name = plugin_basename(__FILE__);
		add_filter("plugin_action_links_{$ezscm_plugin_name}", array(__CLASS__, "plugin_settings_page"));

		// ** ajax **
		// backend
		add_action("wp_ajax_ezscm_backend", array(__CLASS__, "ajax"));
		// frontend
		add_action("wp_ajax_ezscm_frontend", array(__CLASS__, "ajax_frontend"));
		add_action("wp_ajax_nopriv_ezscm_frontend", array(__CLASS__, "ajax_frontend"));

		// language
		load_plugin_textdomain("ezscm", false, dirname(plugin_basename(__FILE__)) . '/languages/');

		if ( ! function_exists( 'is_plugin_active_for_network' ) ) {
			require_once( ABSPATH . '/wp-admin/includes/plugin.php' );
		}
	}

	/**
		admin pages
	**/
	static function admin_menu() {
		// setup pages
		$role = "manage_options";

		add_menu_page(__("ezSCM", "ezscm"), __("ez Schedule Manager", "ezscm"), $role, "ezscm", array(__CLASS__, "page_schedules"), EZSCM_URL . "assets/img/ez-icon.png");
		add_submenu_page("ezscm", __("Schedule settings", "ezscm"), __("Schedule settings", "ezscm"), $role, "ezscm-settings-schedule", array(__CLASS__, "page_settings_schedule"));
		add_submenu_page("ezscm", __("Global settings", "ezscm"), __("Global settings", "ezscm"), $role, "ezscm-settings-global", array(__CLASS__, "page_settings_global"));
		add_submenu_page("ezscm", __("Help / debug", "ezscm"), __("Help / debug", "ezscm"), $role, "ezscm-help", array(__CLASS__, "page_help"));
		add_submenu_page("ezscm", __("Licensing", "ezscm"), __("Licensing", "ezscm"), $role, "ezscm-licensing", array(__CLASS__, "page_licensing"));
	}

	static function page_schedules() {
		require_once(EZSCM_PATH . "ezscm-page-schedules.php");
	}

	static function page_entries() {
		require_once(EZSCM_PATH . "ezscm-page-entries.php");
	}

	static function page_settings_schedule() {
		require_once(EZSCM_PATH . "ezscm-page-settings-schedule.php");
	}

	static function page_settings_global() {
		require_once(EZSCM_PATH . "ezscm-page-settings-global.php");
	}

	static function page_help() {
		require_once(EZSCM_PATH . "ezscm-page-help.php");
	}

	static function page_licensing() {
		require_once(EZSCM_PATH . "ezscm-page-licensing.php");
	}

	/**
		add settings to plugins page
	**/
	static function plugin_settings_page($links) { 
		$settings_link = "<a href='" . admin_url("admin.php") . "?page=ezscm-settings-global'>" . __("Global Settings", "ezscm") . "</a>";
		array_unshift($links, $settings_link);

		$form_settings_link = "<a href='" . admin_url("admin.php") . "?page=ezscm-settings-schedule'>" . __("Schedule Settings", "ezscm") . "</a>";
		array_unshift($links, $form_settings_link);

		return $links; 
	}

	/**
		scripts
	**/
	static function load_scripts($page, $force_load = false) {
		if (!$force_load && $page != "toplevel_page_ezscm" && !strstr($page, "ez-schedule-manager_page")) return;

		if (substr($_SERVER['REMOTE_ADDR'], 0, 4) != '127.' && $_SERVER['REMOTE_ADDR'] != '::1') {
			$licensed = get_option("ezscm_license_activated", 0)==1;

			if (!$licensed) {
				add_action('admin_notices', array( __CLASS__, "show_register_notice" ));
			}
		}

		wp_enqueue_media();

		// styles
		wp_enqueue_style("thickbox");
		wp_enqueue_style("bootstrap-grid", plugins_url("assets/css/bootstrap-grid.min.css", __FILE__));
		wp_enqueue_style("jquery-ui-smoothness", plugins_url("assets/css/jquery-ui-smoothness.min.css", __FILE__));
		wp_enqueue_style("jquerytimepicker-css", plugins_url("assets/css/jquery.timepicker.css", __FILE__));
		wp_enqueue_style("jquery-opentip", plugins_url("assets/css/opentip.css", __FILE__));
		wp_enqueue_style("ezscm-font-awesome", plugins_url("assets/css/font-awesome.min.css", __FILE__));
		wp_enqueue_style("ezscm-css-backend", plugins_url("style-backend.css", __FILE__), array(), EZSCM_VERSION);

		// scripts
		wp_enqueue_script("jquery");
		wp_enqueue_script("jquery-ui-core");
		wp_enqueue_script("jquery-ui-button");
		wp_enqueue_script("jquery-ui-datepicker");
		wp_enqueue_script("jquery-ui-mouse");
		wp_enqueue_script("jquery-ui-widget");
		wp_enqueue_script("jquery-ui-dialog");
		wp_enqueue_script("jquery-ui-draggable");
		wp_enqueue_script("jquery-ui-droppable");
		wp_enqueue_script("jquery-ui-selectable");
		wp_enqueue_script("jquery-ui-sortable");
		wp_enqueue_script("jquery-ui-spinner");
		wp_enqueue_script("jquery-ui-tabs");
		wp_enqueue_script("momentjs", plugins_url("assets/js/moment.locales.min.js", __FILE__));
		wp_enqueue_script("jquerytimepicker", plugins_url("assets/js/jquery.timepicker.min.js", __FILE__), array("jquery"));
		wp_enqueue_script("jquery-opentip", plugins_url("assets/js/opentip-jquery.min.js", __FILE__), array("jquery"));
		wp_enqueue_script("ezscm-backend", plugins_url("backend.js", __FILE__), array("jquery"), EZSCM_VERSION);

		if ($page == "ez-schedule-manager_page_ezscm-settings-schedule" || $page == "ez-schedule-manager_page_ezscm-settings-global") {
			wp_enqueue_script("ezscm-backend-options", plugins_url("backend-options.js", __FILE__), array("jquery"), EZSCM_VERSION);
		}

		wp_localize_script("ezscm-backend", "ezscm_vars", array(
			"contact" => array(
				"name"    => __("Name", "ezscm"),
				"phone"   => __("Phone number", "ezscm"),
				"email"   => __("Email", "ezscm"),
				"message" => __("Message", "ezscm")
			),
			"entry_delete"    => __("Really delete the selected entry?", "ezscm"),
			"schedule_clear"  => __("Really clear all entries in the selected schedule?", "ezscm"),
			"schedule_delete" => __("Really delete the selected schedule?", "ezscm"),
			"yes_no" => array(
				"no"  => __("No", "ezscm"),
				"yes" => __("Yes", "ezscm")
			)
		));
	}

	/**
		ajax
	**/
	static function ajax_frontend() {
		require_once(EZSCM_PATH . "ajax.php");
	}

	// backend
	static function ajax() {
		require_once(EZSCM_PATH . "ajax-admin.php");
	}

	/**
		check for updates
	**/
	static function check_updates() {
		$remote_url    = EZSCM_UPDATE_URL;
		$purchase_code = get_option("ezscm_purchase_code", "");

		if (!empty($purchase_code)) {
			$remote_url .= "&code={$purchase_code}";
		}

		require_once(EZSCM_PATH . "/plugin-updates/plugin-update-checker.php");
		PucFactory::buildUpdateChecker(
			$remote_url,
			__FILE__,
			"ezscm"
		);
	}

	static function show_register_notice() {
		echo '
		<div class="updated">
		  <p>' . __("Hello! Please register your license to receive automatic updates for <strong>ez Schedule Manager</strong>.", "ezscm") . ' <a href="' . admin_url('admin.php') . '?page=ezscm-licensing">' . __("Register license", "ezscm") . '</a></p>
		</div>';
	}
}
EZSCM::init();

/**
	shortcode
**/
class Ezscm_Shortcode {
	static $add_script;

	static function init() {
		add_shortcode('ezscm', array(__CLASS__, 'handle_shortcode'));

		add_action('init', array(__CLASS__, 'register_script'));
		add_action('wp_footer', array(__CLASS__, 'print_script'));
	}

	static function handle_shortcode($atts) {
		self::$add_script = true;

		extract(shortcode_atts(array(
			"id"    => null,
			"name"  => null,
			"theme" => null
		), $atts));

		$id = (int) $id;

		require_once(EZSCM_PATH . "class.ezscm_frontend.php");
		$ezscm = new Ezscm_frontend();

		return $ezscm->get_output($id, $name, $theme);
	}

	static function register_script() {
		wp_register_style('ezscm-css-frontend', plugins_url("style-frontend.css", __FILE__));
	}

	static function print_script() {
		if ( ! self::$add_script )
			return;

		wp_enqueue_style("ezscm-css-frontend");
		wp_add_inline_style("ezscm-css-frontend", get_option("ezscm_custom_css"));
		wp_enqueue_style("jquery-ui-smoothness", plugins_url("assets/css/jquery-ui-smoothness.min.css", __FILE__));

		wp_enqueue_script("jquery");
		wp_enqueue_script("jquery-ui-core");
		wp_enqueue_script("jquery-ui-dialog");

		// momentjs
		//wp_enqueue_script("momentjs", plugins_url("assets/js/moment.min.js", __FILE__));
		// momentjs + locales
		wp_enqueue_script("momentjs", plugins_url("assets/js/moment.locales.min.js", __FILE__));

		if (get_option("ezscm_debug_mode", 0) == 1) {
			wp_enqueue_script("ezscm-frontend", plugins_url("frontend.js", __FILE__), array("jquery"), EZSCM_VERSION);
		}
		else {
			wp_enqueue_script("ezscm-frontend", plugins_url("frontend.min.js", __FILE__), array("jquery"), microtime());
		}

		wp_localize_script("ezscm-frontend", "ezscm_vars", array(
			"ajaxurl" => admin_url("admin-ajax.php"),
			"form_vars" => array(),
			"debug_mode" => get_option("ezscm_debug_mode", 0),
			"noid" => __("No schedule with the given ID found."),
			"yes_no" => array(
				"no"  => __("No", "ezscm"),
				"yes" => __("Yes", "ezscm")
			),
			"override_day_names" => get_option("ezscm_override_day_names", 0),
			"day_names" => get_option("ezscm_day_names", "")
		));
	}
}
Ezscm_Shortcode::init();