jQuery(document).ready(function($) {
	$.fx.speeds._default = 150;

	// init vars / events for each form
	var ezscm_form_vars = [];
	$(".ezscm").each(function() {
		var form    = this;
		var form_id = $(this).data("id");
		ezscm_form_vars[form_id] = $(this).data("vars");
	});

	// get public callbacks at start
	get_callbacks_public();
	$(".spinner-wrapper").show();

	// dialog
	ezscm_dialog_options = {
		autoOpen: false,
		modal: true,
		width: 400
	};

	ezscm_dialogs = [];
	$(".ezscm-dialog").each(function() {
		var ezscm_id = $(this).parents(".ezscm").data("id");
		ezscm_dialogs[ezscm_id] = $(this).dialog(ezscm_dialog_options);
	});

	/**
		ui events
	**/
	// entry selected
	$("body").on("click touchstart", ".schedule-entry.entry-free:not(.nosubmit)", function() {
		var schedule         = $(this).closest(".ezscm");
		var schedule_details = $(schedule).find(".ezscm-details");
		var schedule_id      = $(schedule).data("id");
		var schedule_options = ezscm_form_vars[schedule_id];

		$(".entry-selected").removeClass("entry-selected");
		$(this).addClass("entry-selected");

		// hide messages
		$(schedule).find(".ezscm-message").fadeOut();

		// internal data
		$("#ezscm-schedule-s_id-" + schedule_id).val(schedule_id);
		$("#ezscm-schedule-date_internal-" + schedule_id).val($(this).data("date"));
		$("#ezscm-schedule-time_internal-" + schedule_id).val($(this).data("time"));

		// public date
		var pubDate = moment($(this).data("date"), "YYYY-MM-DD").format(schedule_options.date_format);
		var pubTime = moment($(this).data("time"), "HH:mm:ss").format(schedule_options.time_format);
		$("#ezscm-schedule-callbackdate-" + schedule_id).val(pubDate);
		$("#ezscm-schedule-callbackhour-" + schedule_id).val(pubTime);	

		if (ezscm_dialogs[schedule_id]) {
			ezscm_dialogs[schedule_id].dialog("open");
		}
		else {
			$(schedule_details).fadeIn();
			$('html, body').animate({
		        scrollTop: schedule_details.offset().top - 200
		    }, 1250);
		}
	});

	// browse weeks
	$("[data-action='get_schedule']").on("click touchstart", function() {
		var id = $(this).closest(".ezscm").data("id");
		var target = "#ezscm-schedule-" + id;
		var week = $(this).data("browseweek");
		$(target).find(".ezscm-schedule-wrapper, .ezscm-details").fadeOut();

		get_callbacks_public(week, target, id);
	});

	// submit
	$(".ezscm-form").on("submit", function(e) {
		$(this).find(".ezscm-loading").show();

		// disable submit button temporarily
		$(this).find("input[type='submit']").attr("disabled", "disabled");

		var id       = $(this).find(".s_id").val();
		var data     = "action=submit&id=" + id + "&" + $(this).serialize();
		var entry    = $(".entry-selected");
		var schedule = $(this).closest(".ezscm");

		$.ajax({
			type: "post",
			url: ezscm_vars.ajaxurl,
			data: {
				action: "ezscm_frontend",
				data: data
			},
			success: function(response) {
				$(schedule).find(".ezscm-loading").hide();
				$(schedule).find(".ezscm-message").fadeIn();

				// enable submit button
				$(".ezscm-submit").removeAttr("disabled");

				if (!response) {
					$(schedule).find(".ezscm-message").text("Something went wrong. :(");
						
					return false;
				}

				if (ezscm_vars.debug_mode == 1) console.log(response);

				response = $.parseJSON(response);

				// error occurred
				if (response.error) {
					$(schedule).find(".ezscm-message").html("<p class='text-danger'>" + response.error + "</p>");
					if (typeof Recaptcha !== "undefined") Recaptcha.reload();

					return false;
				}

				// prevent spam
				if (typeof Recaptcha !== "undefined") Recaptcha.reload();

				// success
				if (response.success) {
					$(entry).removeClass("entry-free").addClass("entry-public");
					$(schedule).find(".ezscm-message").text(response.success);
					$(schedule).find(".ezscm-details").fadeOut();
					
					// dialog
					if (typeof ezscm_dialogs[id] !== "undefined") {
						ezscm_dialogs[id].dialog("close");
					}

					// redirect the user
					if (ezscm_form_vars[id] && ezscm_form_vars[id].redirect_url && ezscm_form_vars[id].redirect_url.length > 0) {
						window.location.replace(ezscm_form_vars[id].redirect_url);
					}
				}
			}
		});

		e.preventDefault();
		return false;
	});

	function get_callbacks_public(week, target, id) {
		$(".spinner").fadeIn();
		$(target).find(".ezscm-message").fadeOut();

		if (!target) target = ".ezscm";

		$(target).each(function(i, schedule) {
			if (!id) id = $(schedule).find(".s_id").val();
			
			var postData = "action=get_public_schedule";
			postData    += "&id=" + id;
			postData    += "&add=" + $(schedule).data("add");

			if (week) {
				postData += "&week=" + week;
			}

			$.ajax({
				type: "post",
				url: ezscm_vars.ajaxurl,
				data: {
					action: "ezscm_frontend",
					data: postData
				},
				success: function(response) {
					if (!response) return false;

					if (ezscm_form_vars.debug_mode == 1) console.log(response);

					response = $.parseJSON(response);

					$(schedule).find(".ezscm-loading, .spinner").hide();
					$(schedule).find(".ezscm-container, .ezscm-schedule-wrapper, .ezscm-browse").fadeIn();

					if (response.options.length < 1) {
						$(schedule).find(".ezscm-schedule-wrapper").text(ezscm_vars.noid);
						return false;
					}

					show_schedule($(schedule), response.entries, response.options, week);
				}
			});
		});
	}

	// show schedule entries
	function show_schedule(target, entries, options, week) {
		// set language
		moment.locale(options.lang_dates);

		// parse blocked times
		var days = options.days_available.split(",");

		var blocked = [];
		if (!options.time_block) {
			blocked.push([
				options.time_block_start,
				options.time_block_end
			]);
		}
		else {
			var tmp_blocked_split = options.time_block.split(",");
			var tmp_blocked_array = [];
			for (var i in tmp_blocked_split) {
				tmp_blocked_array.push(tmp_blocked_split[i].split("-"));
			}

			blocked.push(tmp_blocked_array);
		}

		if (!week) {
			week = moment().startOf("isoWeek");
		}
		else {
			week = moment(week, "YYYY-MM-DD");
		}

		// set internal block times
		var	block_begin, block_end, block_diff;
		if (blocked.length > 0) {
			block_begin = moment(blocked[0], "HH:mm");
			block_end   = moment(blocked[1], "HH:mm");
		}

		// closed days
		var closed_days = $.parseJSON(options.closed_days);

		// calculate time steps
		var
			tmp_begin = options.time_begin.split(":"),
			tmp_end   = options.time_end.split(":"),
			tmp_steps = options.time_steps.split(":");

		var
			begin_hour   = parseInt(tmp_begin[0]),
			begin_minute = parseInt(tmp_begin[1]),
			end_hour     = parseInt(tmp_end[0]),
			end_minute   = parseInt(tmp_end[1]),
			step_hour    = parseInt(tmp_steps[0]),
			step_minute  = parseInt(tmp_steps[1]);

		var
			tmp_hours   = (end_hour - begin_hour) * 60,
			tmp_minutes = end_minute - begin_minute;

		var steps      = Math.floor((tmp_hours + tmp_minutes) / (step_hour*60 + step_minute));
		var now        = moment();
		var entry_free = false; // used for skip_week

		// output
		var out = "<div class='ezscm-schedule'>";
		var out_times = "";

		// hours
		out_times += "<ul class='schedule-day schedule-times schedule-times-main'>";
		out_times += "	<li class='schedule-header'></li>";

		var tmp_time = moment({ hours: begin_hour, minute: begin_minute });
		for (var h = 0; h < steps; h++) {
			out_times += "<li class='entry-small'>";
			out_times += "	<span>" + tmp_time.format(options.time_format) + "</span>";
			out_times += "</li>";

			tmp_time.add(step_hour, "hours").add(step_minute, "minutes");
		}
		out_times += "</ul>";

		out += out_times;

		// days
		var days_show = parseInt(options.show_weeks_amount_frontend) * 7;
		for (var day_counter = 0; day_counter < days_show; day_counter++) {
			// start with current weekday
			var day_offset = 0;
			if (options.start_current_weekday == 1) {
				day_offset = moment().weekday() - 1;
			}

			var is_first_day = false;
			var loop_day;
			var day_counter_offset = day_counter + day_offset;

			// reset back to day-range (0 to 7)
			if (day_counter_offset >= 7) {
				var loop_day_calc = day_counter_offset - Math.floor(day_counter_offset / 7) * 7;

				if (!days[loop_day_calc]) continue;

				loop_day = parseInt(days[loop_day_calc]) + day_counter_offset - loop_day_calc;
				is_first_day = days[loop_day_calc] == days[day_counter_offset];
			}
			else {
				if (!days[day_counter_offset]) continue;
				if (days[day_counter_offset] == 0) days[day_counter_offset] = 7;

				loop_day = parseInt(days[day_counter_offset]);
			}

			// add times
			if (is_first_day) {
				out += "</div><div class='ezscm-schedule'>";
				out += out_times;
			}

			// set loop date
			var m = moment(week).startOf("isoWeek").isoWeekday(loop_day).hour(begin_hour).minute(begin_minute);

			var currentDateInternal = moment(week, "YYYY-MM-DD").startOf("isoWeek").isoWeekday(loop_day);
			var currentDate         = currentDateInternal.format("YYYY-MM-DD");
			var currentDay          = currentDateInternal.format(options.date_format);

			// default day name
			var currentDayName = moment(week, "YYYY-MM-DD").isoWeekday(loop_day).format("dddd");
			// check for custom day name
			if (ezscm_vars.override_day_names == 1) {
				var custom_day_names = ezscm_vars.day_names.split(",");

				if (typeof custom_day_names[loop_day] !== "undefined") {
					currentDayName = custom_day_names[loop_day - 1];
				}
			}

			// check if day is closed / blocked
			var closed_day = false;
			for (var ci = 0; ci < closed_days.length; ci++) {
				var closed_day_from = moment(closed_days[ci].from, "YYYY-MM-DD");
				var closed_day_to   = moment(closed_days[ci].to, "YYYY-MM-DD");

				if ((currentDateInternal.isAfter(closed_day_from) && currentDateInternal.isBefore(closed_day_to)) ||
					currentDateInternal.isSame(closed_day_from) || currentDateInternal.isSame(closed_day_to)) {
					closed_day = true;
				}
			}

			out += "<div class='schedule-list-day'>";

			// times + responsive times
			var tmp_time         = moment({ hours: begin_hour, minute: begin_minute });

			out += "<ul class='schedule-day schedule-times schedule-times-responsive'>";
			out += "	<li class='schedule-header'></li>";
			for (var h = 0; h < steps; h++) {
				out += "<li class='entry-small'>";
				out += "	<span>" + tmp_time.format(options.time_format) + "</span>";
				out += "</li>";

				tmp_time.add(step_hour, "hours").add(step_minute, "minutes");
			}
			out += "</ul>";

			out += "<ul class='schedule-times'>";
			out += "	<li class='schedule-header'>" + currentDayName + "<br>" + currentDay + "</li>";

			for (var s = 0; s < steps; s++) {
				var tmp_class;
				var current_data_time = m.format("HH:mm:ss");

				// if day is closed, just add blocked class
				if (closed_day) {
					tmp_class = "entry-blocked";
				}
				else {
					tmp_class                 = "entry-free";
					var current_time_internal = currentDateInternal.hour(m.hour()).minute(m.minute());
					var date_min              = moment().add(options.entry_time_ahead, "hours");

					// check if time is in the past
					if (current_time_internal.isBefore(now) || current_time_internal.isBefore(date_min)) {
						tmp_class = "entry-blocked";
					}

					// fill blocked times
					for (var b = 0; b < blocked.length; b++) {
						var block_begin = moment(currentDate + " " + blocked[b][0]);
						var block_end   = moment(currentDate + " " + blocked[b][1]);

						if ((m.isAfter(block_begin) || m.isSame(block_begin)) && m.isBefore(block_end)) {
							tmp_class = "entry-blocked";
							break;
						}
					}

					if (tmp_class == "entry-free") entry_free = true;

					// check if user can submit anything
					if (options.display_only == 1) tmp_class += " nosubmit";
				}

				out += "<li class='schedule-entry " + tmp_class + "' data-date='" + currentDate + "' data-time='" + current_data_time + "' data-dateformat='" + currentDay + "' data-s_id='" + options.days_available.s_id + "'>";
				out += "	<span></span>";
				out += "</li>";			

				m.add(step_hour, "hours").add(step_minute, "minutes");
			}

			out += "</ul>";
			out += "</div>";
		}

		$(target).find(".ezscm-schedule-wrapper").html(out);

		/**
			fill public entries
		**/
		for (var e = 0; e < entries.length; e++) {
			var	entryDate      = entries[e].date,
				entryTimeBegin = entries[e].time_begin;

			var targetEntry = ".schedule-entry[data-date='" + entryDate + "'][data-time='" + entryTimeBegin + "']";

			$(target).find(targetEntry)
				.removeClass("entry-free")
				.addClass("entry-public");

			if (entries[e].hasOwnProperty("data")) {
				$(target).find(targetEntry + " span").text(entries[e].data);
			}

			entry_free = false;
		}

		if (!entry_free && options.skip_week == 1) {
			var nextweek = moment(week).add(1, "week").format("YYYY-MM-DD");

			get_callbacks_public(nextweek, target);
			return;
		}

		// next / prev week values
		var week_prev = moment(week).subtract(days_show, "days").format("YYYY-MM-DD");
		var week_next = moment(week).add(days_show, "days").format("YYYY-MM-DD");
		
		$(target).find(".ezscm-browse-prev").data("browseweek", week_prev);
		$(target).find(".ezscm-browse-next").data("browseweek", week_next);

		if (moment(week_prev).isBefore(moment().startOf("isoWeek"))) {
			$(target).find(".ezscm-browse-prev").fadeOut();
		}
		else {
			$(target).find(".ezscm-browse-prev").fadeIn();
		}
	}
});